## Catavolt Javascript SDK and React Components  
### Overview  
The Catavolt Javascript solution is comprised of three layers, all independently usable, and all providing a different level of programmatic access and customization opportunies.  
  
1. **The Catavolt Javascript SDK** - A pure Javascript API, providing low-level access to the Catavolt data model and server API.   
1. **The Catavolt Core React Components** - A set of components that provide declarative access to many of the SDK functions, making the construction of custom Catavolt Apps much easier.  These components provide very minimal, default styling, and allow the client to fully control the document structure (i.e. html and css)   
1. **The Catavolt React HTML Components** - A set of components that provide default styling, include some html markup, but still allow for customization and the application of a custom style.  The components are also built so that any single component could be replaced (or wrapped) with a custom component if desired.     

The Catavolt React Component hierarchy mimics the Catavolt Extender (server model) as well as the object model used in the Catvolt SDK. Please see [this document](../componentOverview.pdf).
   
### Catavolt React HTML Components (‘catavolt-html’ package)  
  

**&lt;{@link CvLoginPanel}/&gt;**  
Basic styled, configurable Login Component  
{@link CvLoginPanelProps}

**&lt;{@link CvWorkbenchManager}/&gt;**  
Basic Workbench ‘switcher’ composed of a {@link CvDropdownWorkbenchMenu} and {@link CvGraphicalWorkbench} by default.  ‘Pluggable’ with other implementations of these.  
{@link CvWorkbenchManagerProps}

**&lt;{@link CvGraphicalWorkbench}/&gt;**  
Provides a styled, ‘icon-based’ view of a workbench and launchers, with configurable number of columns.  
{@link CvGraphicalWorkbenchProps}

**&lt;{@link CvDropdownWorkbenchMenu}/&gt;**  
Provides ‘Workbench selection’ menu implementations for selecting an active workbench.   Currently a basic ‘drop-down’ menu  and a ‘Tabbed’ menu are available.  
{@link CvGraphicalWorkbenchMenuProps}

**&lt;{@link CvNavigator}/&gt;**  
Renders a ‘Navigation Request’ using a default, Catavolt styled {@link CvFormPanel}  
{@link CvNavigatorProps}

**&lt;{@link CvWebNavigator}/&gt;**  
Renders a ‘Web Navigation Request’ by opening the url in a browser  
{@link CvWebNavigatorProps}

**&lt;{@link CvFormPanel}/&gt;**  
Builds ‘styled’ subtypes of Form layouts (tabbed, 3-pane, ordered, etc.).  Provides pluggable access for new or custom form layouts.  
{@link CvFormPanel}

**&lt;{@link CvListPanel}/&gt;**  
Simple tabular list renderer.  
{@link CvListPanelProps}

**&lt;{@link CvGridPanel}/&gt;**  
Default tabular list renderer.  
{@link CvGridPanelProps}

**&lt;{@link CvDetailsPanel}/&gt;**  
Default ‘cell’ based detail renderer.  Provides editing/saving support.  Uses {@link CvCellValueDef}’s to render cells.  
{@link CvDetailsPanelProps}

**&lt;{@link CvCellValueDef}/&gt;**  
Render’s a Catavolt ‘Cell Value Definition’.   AttributeCellValueDef, LabelCellValueDef, etc.  
{@link CvCellValueDefProps}

**&lt;{@link CvDataAnno}/&gt;**  
Applies default style to a properties and records, based on the Data Annotations attached to the entities (if any).  
{@link CvDataAnnoProps}

**&lt;{@link CvHeroHeader}/&gt;**  
Convenience component to add a logo or company header  
{@link CvHeroHeaderProps}

**&lt;{@link CvDropdownMenu}/&gt;**  
Renders MenuDef’s as a basic ‘Dropdown’ menu  
{@link CvMenuProps}

**&lt;CvBarcodeScanPanel/&gt;**  
Render a default, styled barcode scan detail panel  

**&lt;CvGeoInfoPanel/&gt;**  
Render a default, styled geo location or geofix detail panel  

**&lt;{@link CvMapPanel}/&gt;**  
Render a default, styled map panel  
{@link CvMapPanelProps}

**&lt;CvCalendarPanel/&gt;**  
(pending implementation) Render a default, styled calendar  

**&lt;{@link CvGraphPanel}/&gt;**  
Render a default, styled graph
{@link CvGraphPanelProps}

**&lt;CvImagePickerPanel/&gt;**  
(pending implementation) Render a default, styled image picker panel  

**&lt;{@link CvMessagePanel}/&gt;**  
Uses a {@link CvMessagePane} to display a "toast-like" notification of messages (based on toast.js)
{@link CvMessagePanelProps}  




