"use strict";
/**
 * Created by rburson on 3/18/16.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var catreact_1 = require("catreact");
var catavolt_sdk_1 = require("catavolt-sdk");
var catreact_html_1 = require("./catreact-html");
var CvMenu_1 = require("./CvMenu");
/*
    Display property definitions
    These are the values that may be expected to be in the CvDisplayProperties prop
 */
var DISPLAY_PROP_ACTIVE_FORM_TAB = 'active_form_tab';
var DISPLAY_PROP_VIEW_ID = 'viewId';
var DISPLAY_PROP_LIST_SELECTION = 'list_sel';
var DISPLAY_PROP_LIST_TOTAL_ROWS = 'list_total_rows';
var DISPLAY_PROP_LIST_FIRST_VISIBLE_ROW = 'list_first_row';
var DISPLAY_PROP_FORM_LAYOUT = 'form_layout';
/**
 * Renders a FormContext, utilizing a series of composite, component renderers
 * These may be overriden via {@link CvFormComponentProvider}
 */
exports.CvFormPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentDidMount: function () {
    },
    getChildContext: function () {
        var ctx = this.getDefaultChildContext();
        ctx.cvContext.scopeCtx.scopeObj = this.formContext();
        return ctx;
    },
    getDefaultProps: function () {
        return {
            formContext: null,
            navigationListeners: [],
            actionListeners: [],
            navTarget: null,
            layoutOverrideElem: null,
            stateChangeListeners: [],
            formComponentProvider: null,
            displayProperties: new catreact_html_1.CvDisplayProperties(),
            displayPropChangeListeners: []
        };
    },
    render: function () {
        var formContext = this.formContext();
        if (formContext) {
            if (this.props.renderer) {
                return this.props.renderer(this.getChildContext().cvContext);
            }
            else {
                var defaultProvider = this._getDefaultFormComponentProvider();
                return React.createElement(catreact_1.CvForm, { formContext: this.formContext(), stateChangeListeners: this.props.stateChangeListeners, childFormComponentRenderer: defaultProvider.getChildFormComponentRenderer(), detailsComponentRenderer: defaultProvider.getDetailsComponentRenderer(), listComponentRenderer: defaultProvider.getListComponentRenderer(), mapComponentRenderer: defaultProvider.getMapComponentRenderer(), printMarkupComponentRenderer: defaultProvider.getPrintMarkupComponentRenderer(), graphComponentRenderer: defaultProvider.getGraphComponentRenderer(), calendarComponentRenderer: defaultProvider.getCalendarComponentRenderer(), barcodeScanComponentRenderer: defaultProvider.getBarcodeScanComponentRenderer(), imagePickerComponentRenderer: defaultProvider.getImagePickerComponentRenderer(), geoFixComponentRenderer: defaultProvider.getGeoFixComponentRenderer(), geoLocationComponentRenderer: defaultProvider.getGeoLocationComponentRenderer(), formRenderer: defaultProvider.getFormRenderer() });
            }
        }
        else {
            return null;
        }
    },
    customFormComponentProvider: function () {
        return this.props.formComponentProvider;
    },
    formContext: function () {
        return this.props.formContext || this.firstInScope(catavolt_sdk_1.FormContext);
    },
    /*
     This (default) component provider checks to see if there is a supplied customFormComponentProvider and
     whether or not it has provided a custom component of a particular type.  If it has provided a component of
     a particular type, the custom component will be returned instead of the default component
     */
    _getDefaultFormComponentProvider: function () {
        var _this = this;
        //override this provider with a custom provider
        var customProvider = this.customFormComponentProvider();
        return {
            getFormRenderer: function () {
                return (customProvider && customProvider.getFormRenderer) ? customProvider.getFormRenderer() :
                    function (cvContext, childComponents) {
                        return _this._getFormLayoutComponent(childComponents);
                    };
            },
            getChildFormComponentRenderer: function () {
                return (customProvider && customProvider.getChildFormComponentRenderer) ? customProvider.getChildFormComponentRenderer() :
                    (function (formContext, stateChangeListener, index) {
                        return React.createElement(exports.CvFormPanel, { formContext: formContext, navigationListeners: _this.props.navigationListeners, stateChangeListeners: [stateChangeListener], actionListeners: _this.props.actionListeners, layoutOverrideElem: _this.props.layoutOverrideElem });
                    });
            },
            getDetailsComponentRenderer: function () {
                return (customProvider && customProvider.getDetailsComponentRenderer) ? customProvider.getDetailsComponentRenderer() :
                    (function (detailsContext, stateChangeListener, index) {
                        var topMenuDef = (detailsContext.menuDefs && detailsContext.menuDefs.length > 0) ? detailsContext.menuDefs[0] : null;
                        var contextMenuDef = topMenuDef ? topMenuDef.findContextMenuDef() : null;
                        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
                        var VIEW_ID_PROP_NAME = detailsContext.paneRef + '_' + DISPLAY_PROP_VIEW_ID;
                        var viewId = _this.props.displayProperties.getProperty(VIEW_ID_PROP_NAME);
                        var menu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: detailsContext, menuDef: contextMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: _this.props.navTarget });
                        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: detailsContext, menuDef: topMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: _this.props.navTarget });
                        var viewMenu = React.createElement(catreact_html_1.CvDropdownViewMenu, { paneContext: detailsContext, selectedViewId: viewId, actionListeners: _this.props.actionListeners, navigationListeners: _this.props.navigationListeners, viewDescs: detailsContext.viewDescs, navTarget: _this.props.navTarget, viewSelectionListener: function (viewId) {
                                _this.props.displayProperties.getAndUpdate(VIEW_ID_PROP_NAME, viewId)
                                    .publishToListeners(_this.props.displayPropChangeListeners);
                            }, openLeft: true });
                        var panel = null;
                        var actionSource = detailsContext.actionSource;
                        if (actionSource instanceof catavolt_sdk_1.ContextAction &&
                            '#search' === actionSource.actionId) {
                            panel = React.createElement(catreact_html_1.CvSearchPanel, { detailsContext: detailsContext, navigationListeners: _this.props.navigationListeners, stateChangeListeners: [stateChangeListener], actionListeners: _this.props.actionListeners, actionProvider: actionHandlerAdapter });
                        }
                        else {
                            panel = React.createElement(catreact_html_1.CvDetailsPanel, { detailsContext: detailsContext, navigationListeners: _this.props.navigationListeners, stateChangeListeners: [stateChangeListener], actionListeners: _this.props.actionListeners, actionProvider: actionHandlerAdapter });
                        }
                        //using unique key here based on the viewId, hints to React that this this be a new instance of this component
                        var key = viewId ? detailsContext.paneRef + '_' + viewId : detailsContext.paneRef;
                        return React.createElement(exports.CvFormItemPanel, { paneContext: detailsContext, displayElement: panel, menuElement: menu, menuElementClassName: "cv-panel-menu-item-details", toolbarElement: toolbarMenu, toolbarElementClassName: "cv-panel-heading-toolbar-details", viewMenuElement: viewMenu, key: key, index: index });
                    });
            },
            getGraphComponentRenderer: function () {
                return (customProvider && customProvider.getGraphComponentRenderer) ? customProvider.getGraphComponentRenderer() :
                    (function (graphContext, stateChangeListener, index) {
                        var topMenuDef = (graphContext.menuDefs && graphContext.menuDefs.length > 0) ? graphContext.menuDefs[0] : null;
                        var contextMenuDef = topMenuDef ? topMenuDef.findContextMenuDef() : null;
                        var selectionAdapter = new catreact_1.CvValueAdapter();
                        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
                        var menu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: graphContext, menuDef: contextMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], navTarget: _this.props.navTarget, actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter });
                        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: graphContext, menuDef: topMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter, navTarget: _this.props.navTarget });
                        var panel = React.createElement(catreact_html_1.CvGraphPanel, { graphContext: graphContext, navigationListeners: _this.props.navigationListeners, selectionListener: selectionAdapter.createValueListener(), actionProvider: actionHandlerAdapter, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener] });
                        return React.createElement(exports.CvFormItemPanel, { paneContext: graphContext, displayElement: panel, menuElement: menu, toolbarElement: toolbarMenu, index: index });
                    });
            },
            getListComponentRenderer: function () {
                return (customProvider && customProvider.getListComponentRenderer) ? customProvider.getListComponentRenderer() :
                    (function (listContext, stateChangeListener, index) {
                        var displayProps = _this.props.displayProperties;
                        var topMenuDef = (listContext.menuDefs && listContext.menuDefs.length > 0) ? listContext.menuDefs[0] : null;
                        var contextMenuDef = topMenuDef ? topMenuDef.findContextMenuDef() : null;
                        var selectionAdapter = new catreact_1.CvValueAdapter();
                        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
                        var VIEW_ID_PROP_NAME = listContext.paneRef + '_' + DISPLAY_PROP_VIEW_ID;
                        var viewId = displayProps.getProperty(VIEW_ID_PROP_NAME);
                        var LIST_SELECTION_PROP_NAME = listContext.paneRef + '_' + DISPLAY_PROP_LIST_SELECTION;
                        var selectedItems = displayProps.getProperty(LIST_SELECTION_PROP_NAME);
                        var NUM_ROWS_PROP_NAME = listContext.paneRef + '_' + DISPLAY_PROP_LIST_TOTAL_ROWS;
                        var initialRows = displayProps.getProperty(NUM_ROWS_PROP_NAME) ? Number(displayProps.getProperty(NUM_ROWS_PROP_NAME)) : null;
                        var FIRST_ROW_PROP_NAME = listContext.paneRef + '_' + DISPLAY_PROP_LIST_FIRST_VISIBLE_ROW;
                        var firstVisibleRow = displayProps.getProperty(FIRST_ROW_PROP_NAME) ? Number(displayProps.getProperty(FIRST_ROW_PROP_NAME)) : null;
                        var menu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: listContext, menuDef: contextMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], navTarget: _this.props.navTarget, actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter });
                        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: listContext, menuDef: topMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter, navTarget: _this.props.navTarget });
                        var viewMenu = React.createElement(catreact_html_1.CvDropdownViewMenu, { paneContext: listContext, selectedViewId: viewId, actionListeners: _this.props.actionListeners, navigationListeners: _this.props.navigationListeners, viewDescs: listContext.viewDescs, navTarget: _this.props.navTarget, viewSelectionListener: function (viewId) {
                                //new view, reset all display props
                                _this.props.displayProperties.getAndRemove(NUM_ROWS_PROP_NAME)
                                    .getAndRemove(LIST_SELECTION_PROP_NAME)
                                    .getAndRemove(FIRST_ROW_PROP_NAME)
                                    .getAndUpdate(VIEW_ID_PROP_NAME, viewId)
                                    .publishToListeners(_this.props.displayPropChangeListeners);
                            }, openLeft: true });
                        var selectionAdapterDelegate = selectionAdapter.createValueListener(); //use this to notify the adapter, which in turn notifies it's subscribers (ie the menu)
                        var gridListener = function (event) {
                            var displayProps = _this.props.displayProperties;
                            if (event.type === catreact_html_1.CvGridEventType.SELECTION_EVENT) {
                                var selections = event.eventObj;
                                selectionAdapterDelegate(selections); //notify the menu
                                displayProps.getAndUpdate(LIST_SELECTION_PROP_NAME, selections).publishToListeners(_this.props.displayPropChangeListeners);
                            }
                            else if (event.type === catreact_html_1.CvGridEventType.TOTAL_ROWS_LOADED_EVENT) {
                                displayProps.getAndUpdate(NUM_ROWS_PROP_NAME, String(event.eventObj)).publishToListeners(_this.props.displayPropChangeListeners);
                            }
                            else if (event.type === catreact_html_1.CvGridEventType.FIRST_VISIBLE_ROW_CHANGE_EVENT) {
                                displayProps.getAndUpdate(FIRST_ROW_PROP_NAME, String(event.eventObj)).publishToListeners(_this.props.displayPropChangeListeners);
                            }
                        };
                        //const panel = <CvListPanel listContext={listContext}
                        var panel = React.createElement(catreact_html_1.CvGridPanel, { listContext: listContext, initialSelectedItems: selectedItems, numInitialRows: initialRows, initialFirstVisibleRow: firstVisibleRow, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, gridListener: gridListener, actionProvider: actionHandlerAdapter, stateChangeListeners: [stateChangeListener], searchable: contextMenuDef && !!(contextMenuDef.findAtId('#search')) });
                        //using unique key here based on the viewId, hints to React that this this be a new instance of this component
                        var key = viewId ? listContext.paneRef + '_' + viewId : listContext.paneRef;
                        return React.createElement(exports.CvFormItemPanel, { paneContext: listContext, displayElement: panel, menuElement: menu, toolbarElement: toolbarMenu, viewMenuElement: viewMenu, key: key, index: index });
                    });
            },
            getMapComponentRenderer: function () {
                return (customProvider && customProvider.getMapComponentRenderer) ? customProvider.getMapComponentRenderer() :
                    (function (mapContext, stateChangeListener, index) {
                        var topMenuDef = (mapContext.menuDefs && mapContext.menuDefs.length > 0) ? mapContext.menuDefs[0] : null;
                        var contextMenuDef = topMenuDef ? topMenuDef.findContextMenuDef() : null;
                        var selectionAdapter = new catreact_1.CvValueAdapter();
                        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
                        var menu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: mapContext, menuDef: contextMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], navTarget: _this.props.navTarget, actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter });
                        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: mapContext, menuDef: topMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), selectionProvider: selectionAdapter, navTarget: _this.props.navTarget });
                        var panel = React.createElement(catreact_html_1.CvMapPanel, { mapContext: mapContext, navigationListeners: _this.props.navigationListeners, selectionListener: selectionAdapter.createValueListener(), actionProvider: actionHandlerAdapter, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener] });
                        return React.createElement(exports.CvFormItemPanel, { paneContext: mapContext, displayElement: panel, menuElement: menu, toolbarElement: toolbarMenu, index: index });
                    });
            },
            getPrintMarkupComponentRenderer: function () {
                return (customProvider && customProvider.getPrintMarkupComponentRenderer) ? customProvider.getPrintMarkupComponentRenderer() :
                    (function (printMarkupContext, stateChangeListener, index) {
                        var topMenuDef = (printMarkupContext.menuDefs && printMarkupContext.menuDefs.length > 0) ? printMarkupContext.menuDefs[0] : null;
                        var contextMenuDef = topMenuDef ? topMenuDef.findContextMenuDef() : null;
                        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
                        var menu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: printMarkupContext, menuDef: contextMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: _this.props.navTarget });
                        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: printMarkupContext, menuDef: topMenuDef, openLeft: true, navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, stateChangeListeners: [stateChangeListener], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: _this.props.navTarget });
                        var panel = React.createElement(catreact_html_1.CvPrintPanel, { printMarkupContext: printMarkupContext, navigationListeners: _this.props.navigationListeners, stateChangeListeners: [stateChangeListener], actionListeners: _this.props.actionListeners, actionProvider: actionHandlerAdapter });
                        return React.createElement(exports.CvFormItemPanel, { paneContext: printMarkupContext, displayElement: panel, menuElement: menu, toolbarElement: toolbarMenu, index: index });
                    });
            },
            /**
             * Pending implementation
             * @private
             */
            getCalendarComponentRenderer: function () {
                return (customProvider && customProvider.getCalendarComponentRenderer) ? customProvider.getCalendarComponentRenderer() :
                    (function (calendarContext, stateChangeListener, index) {
                        return React.createElement("h2", null, "There is not yet a default Calendar Implementation. You can provide one, if you like.");
                    });
            },
            /**
             * Pending implementation
             * @private
             */
            getImagePickerComponentRenderer: function () {
                return (customProvider && customProvider.getImagePickerComponentRenderer) ? customProvider.getImagePickerComponentRenderer() :
                    (function (imagePickerContext, stateChangeListener, index) {
                        return React.createElement("h2", null, "There is not yet a default ImagePicker Implementation. You can provide one, if you like.");
                    });
            },
            /**
             * Pending implementation
             * @private
             */
            getBarcodeScanComponentRenderer: function () {
                return (customProvider && customProvider.getBarcodeScanComponentRenderer) ? customProvider.getBarcodeScanComponentRenderer() :
                    (function (barcodeScanContext, stateChangeListener, index) {
                        return React.createElement("h2", null, "There is not yet a default BarcodeScan Implementation. You can provide one, if you like.");
                    });
            },
            /**
             * Pending implementation
             * @private
             */
            getGeoFixComponentRenderer: function () {
                return (customProvider && customProvider.getGeoFixComponentRenderer) ? customProvider.getGeoFixComponentRenderer() :
                    (function (geoFixContext, stateChangeListener, index) {
                        return React.createElement("h2", null, "There is not yet a default GeoFix Implementation. You can provide one, if you like.");
                    });
            },
            /**
             * Pending implementation
             * @private
             */
            getGeoLocationComponentRenderer: function () {
                return (customProvider && customProvider.getGeoLocationComponentRenderer) ? customProvider.getGeoLocationComponentRenderer() :
                    (function (geoLocationContext, stateChangeListener, index) {
                        return React.createElement("h2", null, "There is not yet a default GeoLocation Implementation. You can provide one, if you like.");
                    });
            }
        };
    },
    /**
     *
     * @param childComponents These are the components which have been rendered by the various child component functions
     * @returns {any}
     * @private
     */
    _getFormLayoutComponent: function (childComponents) {
        var formContext = this.formContext();
        var formDef = this.formContext().formDef;
        var displayProperties = this.props.displayProperties;
        var formLayoutType = displayProperties.getProperty(DISPLAY_PROP_FORM_LAYOUT);
        if (!formLayoutType) {
            if (formDef.isTabsLayout) {
                formLayoutType = 'tab';
            }
            else if (formDef.isThreeBoxOneLeftLayout) {
                formLayoutType = 'rsplit';
            }
            else if (formDef.isThreeBoxOneRightLayout) {
                formLayoutType = 'lsplit';
            }
            else if (formDef.isThreeBoxOneOverLayout) {
                formLayoutType = 'bsplit';
            }
            else if (formDef.isThreeBoxOneUnderLayout || formDef.isTwoVerticalLayout) {
                formLayoutType = 'tsplit';
            }
            else if (formDef.isFourBoxSquareLayout) {
                formLayoutType = '4box';
            }
            else if (formDef.isOptionsFormLayout) {
                formLayoutType = 'opt';
            }
            else {
                formLayoutType = 'vert';
            }
        }
        var component = null;
        if (this.props.layoutOverrideElem) {
            var props = {
                formContext: formContext,
                formComponents: childComponents,
                navigationListeners: this.props.navigationListeners,
                actionListeners: this.props.actionListeners,
                navTarget: this.props.navTarget
            };
            component = React.createElement(this.props.layoutOverrideElem, props);
        }
        else if (formLayoutType === 'tab') {
            component = React.createElement(exports.CvTabbedFormPanel, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget, displayProperties: displayProperties, displayPropChangeListeners: this.props.displayPropChangeListeners });
        }
        else if (formLayoutType === 'rsplit') {
            component = React.createElement(exports.CvFormRightSplitPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else if (formLayoutType == 'lsplit') {
            component = React.createElement(exports.CvFormLeftSplitPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else if (formLayoutType == 'bsplit') {
            component = React.createElement(exports.CvFormBottomSplitPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else if (formLayoutType == 'tsplit') {
            component = React.createElement(exports.CvFormTopSplitPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else if (formLayoutType == '4box') {
            component = React.createElement(exports.CvFormFourBoxSplitPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else if (formLayoutType == 'opt') {
            component = React.createElement(exports.CvFormOptionsLayoutPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        else {
            component = React.createElement(exports.CvVerticalLayoutFormPane, { formContext: formContext, formComponents: childComponents, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, navTarget: this.props.navTarget });
        }
        var topMenuDef = (formContext.menuDefs && formContext.menuDefs.length > 0) ? formContext.menuDefs[0] : null;
        var menuDef = (formContext.menuDefs && formContext.menuDefs.length > 0) ? formContext.menuDefs[0].findContextMenuDef() : null;
        var actionHandlerAdapter = new catreact_1.CvValueAdapter();
        var layoutMenu = React.createElement(exports.CvFormLayoutMenu, { selectedItem: displayProperties.getProperty(DISPLAY_PROP_FORM_LAYOUT), displayProperties: displayProperties, displayPropChangeListeners: this.props.displayPropChangeListeners, openLeft: true });
        var viewMenu = React.createElement(catreact_html_1.CvDropdownViewMenu, { paneContext: formContext, actionListeners: this.props.actionListeners, navigationListeners: this.props.navigationListeners, navTarget: this.props.navTarget, viewDescs: formContext.viewDescs, openLeft: true });
        var formMenu = React.createElement(catreact_html_1.CvDropdownMenu, { paneContext: formContext, menuDef: menuDef, openLeft: false, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, stateChangeListeners: [], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: this.props.navTarget });
        var toolbarMenu = React.createElement(CvMenu_1.CvToolbarMenu, { paneContext: formContext, menuDef: topMenuDef, openLeft: true, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, stateChangeListeners: [], actionHandler: actionHandlerAdapter.createValueListener(), navTarget: this.props.navTarget });
        var paneTitle = this.findPaneTitle(formContext);
        return (React.createElement("div", { className: "cv-form" },
            React.createElement("div", { className: "row cv-form-header" },
                React.createElement("div", { style: { display: "inline" } },
                    React.createElement("span", { className: "pull-left cv-form-title" }, paneTitle && paneTitle !== 'Default' ? paneTitle : ''),
                    React.createElement("div", { className: "pull-left cv-form-header-toolbar" }, toolbarMenu),
                    (function () {
                        return menuDef == null ? null :
                            React.createElement("button", { className: "pull-left btn btn-xs cv-toolbar-button", type: "button", "data-toggle": "dropdown", "aria-haspopup": "true", "aria-expanded": "true" },
                                React.createElement("div", { className: "cv-form-menu pull-left" }, formMenu));
                    })()),
                React.createElement("div", { className: "cv-form-menu pull-right" }, viewMenu),
                React.createElement("div", { className: "cv-form-menu pull-right" }, layoutMenu)),
            React.createElement("div", { className: "cv-form-wrapper" }, component)));
    }
});
exports.CvFormLayoutMenu = React.createClass({
    getDefaultProps: function () {
        return {
            selectedItem: null,
            displayProperties: null,
            displayPropChangeListeners: [],
            openLeft: false
        };
    },
    render: function () {
        var _this = this;
        var layouts = ['tab', 'rsplit', 'lsplit', 'bsplit', 'tsplit', '4box', 'vert'];
        var layoutItemClassName = "dropdown-menu cv-dropdown-menu";
        if (this.props.openLeft)
            layoutItemClassName += ' dropdown-menu-right';
        return React.createElement("span", { className: "dropdown cv-layout-dropdown" },
            React.createElement("button", { className: "btn btn-xs", type: "button", "data-toggle": "dropdown", "aria-haspopup": "true", "aria-expanded": "true" },
                React.createElement("span", { className: "glyphicon glyphicon-th", "aria-hidden": "true" }),
                React.createElement("span", { className: "caret" })),
            React.createElement("ul", { className: layoutItemClassName }, layouts.map(function (layout) {
                return React.createElement("li", { onClick: function () {
                        _this.props.displayProperties.getAndUpdate(DISPLAY_PROP_FORM_LAYOUT, layout)
                            .publishToListeners(_this.props.displayPropChangeListeners);
                    } },
                    React.createElement("a", { className: "cv-target" },
                        React.createElement("div", { className: 'cv-layout-item cv-layout-' + layout })));
            })));
    }
});
exports.CvFormItemPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentWillMount: function () {
        if (!this.props.viewMenuElement) {
            this.setState({ viewLoaded: 'default_view' });
        }
    },
    getDefaultProps: function () {
        return { displayElement: null, paneContext: null, menuElement: null, viewMenuElement: null, toolbarElement: null, initOpen: true, index: 1 };
    },
    getInitialState: function () {
        return { viewLoaded: null };
    },
    render: function () {
        if (this.props.paneContext && this.props.displayElement) {
            var paneId = this.props.paneContext.paneDef.paneId;
            var zindex = 100 - this.props.index;
            var menuClassName = "pull-right cv-panel-menu-item " + (this.props.menuElementClassName ? this.props.menuElementClassName : "");
            var toolClassName = "pull-right cv-panel-heading-toolbar " + (this.props.toolbarElementClassName ? this.props.toolbarElementClassName : "");
            return (React.createElement("div", { className: "cv-panel panel panel-primary animated fadeIn", style: { position: 'relative', zIndex: zindex } },
                React.createElement("div", { className: "cv-panel-heading panel-heading" },
                    React.createElement("div", { className: "cv-panel-heading-title" },
                        React.createElement("span", { className: "cv-panel-title" }, this.findPaneTitle(this.props.paneContext))),
                    React.createElement("div", { className: "cv-panel-heading-menu" },
                        React.createElement("div", { className: menuClassName }, this.props.menuElement),
                        React.createElement("div", { className: "pull-right cv-panel-menu-view" }, this.props.viewMenuElement ? React.cloneElement(this.props.viewMenuElement, { viewLoadedListener: this._viewLoaded }) : null),
                        React.createElement("div", { className: toolClassName }, this.props.toolbarElement))),
                React.createElement("div", { className: 'cv-panel-collapse ' + (this.props.initOpen ? 'collapse in' : 'collapse'), id: paneId },
                    React.createElement("div", { className: "cv-panel-body" }, this.state.viewLoaded ? this.props.displayElement : null))));
        }
        else {
            return null;
        }
    },
    _viewLoaded: function (viewId) {
        if (!this.state.viewLoaded || this.state.viewLoaded != viewId) {
            this.setState({ viewLoaded: viewId });
        }
    }
});
exports.CvFormLayout = {
    displayProperties: function () {
        return this.props.displayProperties;
    },
    formComponents: function () {
        return this.props.formComponents;
    },
    formContext: function () {
        return this.props.formContext;
    }
};
exports.CvTabbedFormMenu = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], tabSelectionListener: null, formContext: null, selectedItem: 0 };
    },
    render: function () {
        var _this = this;
        if (this.formComponents() && this.formComponents().length > 0) {
            return React.createElement("ul", { className: "nav nav-pills" }, this.formContext().childrenContexts.map(function (childContext, i) {
                var title = _this.findPaneTitle(childContext);
                if (!title || title.length < 1) {
                    title = 'View ' + (i + 1);
                }
                return React.createElement("li", { onClick: _this._itemSelected.bind(_this, i), role: "presentation", key: i, className: i == _this.props.selectedItem ? 'active' : '' },
                    React.createElement("a", { className: "cv-target" }, title));
            }));
        }
        else {
            return null;
        }
    },
    _itemSelected: function (item) {
        this.props.tabSelectionListener(item);
    }
});
/* Tabbed Panel */
exports.CvTabbedFormPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null,
            displayProperties: new catreact_html_1.CvDisplayProperties(), displayPropChangeListeners: [] };
    },
    render: function () {
        var _this = this;
        if (this.formComponents() && this.formComponents().length > 0) {
            if (this.formComponents().length > 1) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "cv-tab-form-menu" },
                        React.createElement(exports.CvTabbedFormMenu, { formContext: this.formContext(), formComponents: this.formComponents(), selectedItem: this.selectedItem(), tabSelectionListener: function (selectedItem) {
                                _this.displayProperties().getAndUpdate(DISPLAY_PROP_ACTIVE_FORM_TAB, selectedItem)
                                    .publishToListeners(_this.props.displayPropChangeListeners);
                            } })),
                    React.createElement("div", { className: "cv-tab-form-body" }, this.selectedElement())));
            }
            else {
                return React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, this.formComponents()[0])));
            }
        }
        else {
            return null;
        }
    },
    selectedElement: function () {
        return this.formComponents()[this.selectedItem()];
    },
    selectedContext: function () {
        return this.formContext().childrenContexts[this.selectedItem()];
    },
    selectedItem: function () {
        var activeFormTab = this.displayProperties().getProperty(DISPLAY_PROP_ACTIVE_FORM_TAB);
        return activeFormTab ? Number(activeFormTab) : 0;
    }
});
/* Options Form Layout */
exports.CvFormOptionsLayoutPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-option-pane" }, formComponents[0])),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 1; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row" },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* 3, 1 left Layout */
exports.CvFormRightSplitPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[0]),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" },
                            React.createElement("div", { className: "row cv-form-row-split" },
                                React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[1])),
                            React.createElement("div", { className: "row cv-form-row-split" },
                                React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents.length > 2 ? formComponents[2] : null)))),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 3; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row" },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* 3, 1 right Layout */
exports.CvFormLeftSplitPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" },
                            React.createElement("div", { className: "row cv-form-row-split" },
                                React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0])),
                            React.createElement("div", { className: "row cv-form-row-split" },
                                React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[1]))),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents.length > 2 ? formComponents[2] : null)),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 3; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row" },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* 3, 1 over Layout */
exports.CvFormBottomSplitPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0])),
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[1]),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents.length > 2 ? formComponents[2] : null)),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 3; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row" },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* 3, 1 under Layout */
exports.CvFormTopSplitPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[0]),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[1])),
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents.length > 2 ? formComponents[2] : null)),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 3; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row" },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* 4 box Layout */
exports.CvFormFourBoxSplitPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        var formComponents = this.formComponents();
        if (formComponents && formComponents.length > 0) {
            if (formComponents.length < 2) {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[0]))));
            }
            else {
                return (React.createElement("div", { className: "cv-form-container" },
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[0]),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents[1])),
                    React.createElement("div", { className: "row cv-form-row-split" },
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents.length > 2 ? formComponents[2] : null),
                        React.createElement("div", { className: "col-sm-6 cv-form-col" }, formComponents.length > 3 ? formComponents[3] : null)),
                    React.createElement("div", { className: "row cv-form-row" },
                        React.createElement("div", { className: "col-sm-12 cv-form-col" }, (function () {
                            var elems = [];
                            for (var i = 4; i < formComponents.length; i++) {
                                elems.push(React.createElement("div", { className: "row cv-form-row", key: i },
                                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, formComponents[i])));
                            }
                            return elems;
                        })()))));
            }
        }
        else {
            return null;
        }
    },
});
/* Vertical Layout */
exports.CvVerticalLayoutFormPane = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvFormLayout],
    getDefaultProps: function () {
        return { formComponents: [], formContext: null };
    },
    render: function () {
        if (this.formComponents() && this.formComponents().length > 0) {
            return React.createElement("div", { className: "cv-form-container" }, this.formComponents().map(function (formComponent, num) {
                return React.createElement("div", { className: "row cv-form-row" },
                    React.createElement("div", { className: "col-sm-12 cv-form-col" }, (formComponent.key) ? formComponent : React.cloneElement(formComponent, { key: num })));
            }));
        }
        else {
            return null;
        }
    },
});
