"use strict";
/**
 * Created by rburson on 4/27/16.
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var catreact_1 = require("catreact");
var catavolt_sdk_1 = require("catavolt-sdk");
var recharts_1 = require("recharts");
var CV_GRADIENT_COLORS = [
    ['#A1AECF', '#47447A'],
    ['#EF7651', '#994C34'],
    ['#E9C836', '#AA9127'],
    ['#6FB35F', '#497B54'],
    ['#BA9886', '#AE775B'],
    ['#8B8989', '#8B6969'],
    ['#9400D3', '#4B0082'],
    ['#FFC1C1', '#EEB4B4'],
    ['#FFFF00', '#EEEE00'],
    ['#7FFFD4', '#76EEC6'],
    ['#CAE1FF', '#BCD2EE'],
    ['#FF6A6A', '#EE6363'],
    ['#FFD700', '#EEC900'],
    ['#00FF00', '#32CD32'],
    ['#1C86EE', '#104E8B'],
    ['#FF3030', '#EE3B3B'],
    ['#EEE685', '#BDB76B'],
    ['#ADFF2F', '#A2CD5A'],
    ['#EE00EE', '#CD00CD'],
    ['#00F5FF', '#00E5EE'],
    ['#FFFACD', '#EEE8AA'],
    ['#54FF9F', '#2E8B57'],
    ['#4876FF', '#436EEE'],
    ['#FFAEB9', '#CD8C95'],
    ['#EEB422', '#CD9B1D'],
    ['#7FFF00', '#76EE00'],
    ['#6959CD', '#473C8B'],
    ['#FFBBFF', '#EEAEEE'],
    ['#EEEE00', '#CDCD00'],
    ['#00FF00', '#32CD32'],
    ['#AB82FF', '#9F79EE'],
    ['#EE7AE9', '#DA70D6'],
    ['#FFF68F', '#CDC673'],
    ['#76EEC6', '#66CDAA'],
    ['#0000FF', '#0000EE'],
    ['#EE82EE', '#CD96CD'],
    ['#FFE7BA', '#F5DEB3'],
    ['#008B8B', '#008080']
]; // DarkCyan   Trq
// const CV_GRAPH_DEFAULT_COLORS =  ['#1E6FF2', '#F29500', '#15BFA2', '#6F5F5A'];
var CV_GRAPH_DEFAULT_COLORS = CV_GRADIENT_COLORS.map(function (g) { return g[1]; }); // Convert from gradient to flat
/* Base mixin for Charts */
// http://recharts.org/api/#/en-US/api
// https://www.w3.org/TR/SVG/styling.html   For styling
exports.CvChart = {
    axisType: function (graphContext, propName, data) {
        var propDef = graphContext.propDefAtName(propName);
        var numeric = true;
        if (propDef) {
            numeric = propDef.isNumericType;
        }
        else {
            //fallback to a sample value
            numeric = typeof this._getSampleValue(data, propName) === 'number';
        }
        return numeric ? 'number' : 'category';
    },
    colorForDataPointDef: function (dataPointDef, i) {
        return this._zeroFill(dataPointDef.seriesColor) || this.props.defaultSeriesColors[i % this.props.defaultSeriesColors.length];
    },
    getMetaValues: function (graphDef, data) {
        var minX = graphDef.xAxisRangeFrom ? graphDef.xAxisRangeFrom : 0;
        var maxX = graphDef.xAxisRangeTo ? graphDef.xAxisRangeTo : 'auto';
        var minY = graphDef.yAxisRangeFrom ? graphDef.yAxisRangeFrom : 0;
        var maxY = graphDef.yAxisRangeTo ? graphDef.yAxisRangeTo : 'auto';
        var xAxisLabel = graphDef.xAxisLabel;
        var yAxisLabel = graphDef.yAxisLabel;
        return { minX: minX, maxX: maxX, minY: minY, maxY: maxY, xAxisLabel: xAxisLabel, yAxisLabel: yAxisLabel };
    },
    handleClick: function (id) {
        if (this.props.clickHandler) {
            this.props.clickHandler(id);
        }
    },
    _applyAlpha: function (c, a) {
        return catreact_1.ColorUtil.toColorFromNumWithAlpha(Number("0x" + c.substr(1)), a);
    },
    _getSampleValue: function (data, name) {
        if (data && data.length > 0) {
            return data[0][name];
        }
    },
    _zeroFill: function (color) {
        if (color && color.length > 1) {
            color = color.charAt(0) === '#' ? color.substring(1) : color;
            while (color.length < 6) {
                color = '0' + color;
            }
            return '#' + color;
        }
        return null;
    }
};
/* Cartesian style line chart */
exports.CvLineChart = React.createClass({
    mixins: [exports.CvChart],
    render: function () {
        var _this = this;
        var graphContext = this.props.graphContext;
        var graphDef = graphContext.graphDef;
        var _a = this.getMetaValues(graphDef, this.props.data), minX = _a.minX, maxX = _a.maxX, minY = _a.minY, maxY = _a.maxY, xAxisLabel = _a.xAxisLabel, yAxisLabel = _a.yAxisLabel;
        var yPropDef = graphContext.propDefAtName(graphDef.dataPointDefs[0].name); // First will do
        var xType = this.axisType(this.props.graphContext, graphDef.identityDataPointDef.name, this.props.data);
        var yType = this.axisType(this.props.graphContext, graphDef.dataPointDefs[0].name, this.props.data);
        var yAxisFormat = function (value) {
            return catavolt_sdk_1.PropFormatter.formatValueForRead(value, yPropDef);
        };
        return (React.createElement("div", { className: "cv-component-container cv-graph-container" },
            React.createElement(recharts_1.ResponsiveContainer, { width: "99%", height: "95%" },
                React.createElement(recharts_1.LineChart, { data: this.props.data, margin: { top: 20, right: 50, left: 20, bottom: 5 } },
                    React.createElement(recharts_1.XAxis, { dataKey: this.props.identPropName, domain: [minX, maxX], label: xAxisLabel, type: xType }),
                    React.createElement(recharts_1.YAxis, { tickFormatter: yAxisFormat, domain: [minY, maxY], label: yAxisLabel, allowDataOverflow: true, type: yType }),
                    React.createElement(recharts_1.CartesianGrid, { strokeDasharray: "3 3" }),
                    React.createElement(recharts_1.Tooltip, null),
                    React.createElement(recharts_1.Legend, null),
                    graphDef.dataPointDefs.map(function (dataPointDef, i) {
                        var seriesColor = _this.colorForDataPointDef(dataPointDef, i);
                        var legendKey = dataPointDef.legendKey || dataPointDef.name;
                        return React.createElement(recharts_1.Line, { key: i, type: "monotone", dataKey: dataPointDef.name, name: legendKey, dot: true, activeDot: true, stroke: seriesColor, onClick: function (data) { _this.handleClick(data.id); } });
                    })))));
    },
});
exports.CvBarChart = React.createClass({
    mixins: [exports.CvChart],
    render: function () {
        var _this = this;
        var graphContext = this.props.graphContext;
        var graphDef = graphContext.graphDef;
        var _a = this.getMetaValues(graphDef, this.props.data), minX = _a.minX, maxX = _a.maxX, minY = _a.minY, maxY = _a.maxY, xAxisLabel = _a.xAxisLabel, yAxisLabel = _a.yAxisLabel;
        var yPropDef = graphContext.propDefAtName(graphDef.dataPointDefs[0].name); // First will do
        var xType = this.axisType(this.props.graphContext, graphDef.identityDataPointDef.name, this.props.data);
        var yType = this.axisType(this.props.graphContext, graphDef.dataPointDefs[0].name, this.props.data);
        var yAxisFormat = function (value) {
            return catavolt_sdk_1.PropFormatter.formatValueForRead(value, yPropDef);
        };
        return (React.createElement("div", { className: "cv-graph-container" },
            React.createElement(recharts_1.ResponsiveContainer, { width: "99%", height: "95%" },
                React.createElement(recharts_1.BarChart, { data: this.props.data, margin: { top: 20, right: 50, left: 20, bottom: 5 } },
                    React.createElement(recharts_1.XAxis, { dataKey: this.props.identPropName, domain: [minX, maxX], label: xAxisLabel }),
                    React.createElement(recharts_1.YAxis, { tickFormatter: yAxisFormat, domain: [minY, maxY], label: yAxisLabel }),
                    React.createElement(recharts_1.CartesianGrid, { strokeDasharray: "3 3" }),
                    React.createElement(recharts_1.Tooltip, null),
                    React.createElement(recharts_1.Legend, null),
                    graphDef.dataPointDefs.map(function (dataPointDef, i1) {
                        var legendKey = dataPointDef.legendKey || dataPointDef.name;
                        var seriesColor = _this.colorForDataPointDef(dataPointDef, i1);
                        var props = {
                            dataKey: dataPointDef.name,
                            name: legendKey,
                            fill: seriesColor,
                            onClick: function (data) { _this.handleClick(data.id); },
                            key: i1
                        };
                        if (_this.props.stack)
                            props['stackId'] = 'a';
                        return React.createElement(recharts_1.Bar, __assign({}, props), _this.props.data.map(function (data, i2) {
                            var c = data.annoColor ? data.annoColor : seriesColor;
                            return React.createElement(recharts_1.Cell, { key: "cell-" + i2, fill: c });
                        }));
                    })))));
    }
});
exports.CvScatterChart = React.createClass({
    mixins: [exports.CvChart],
    render: function () {
        var _this = this;
        var graphDef = this.props.graphContext.graphDef;
        var _a = this.getMetaValues(graphDef, this.props.data), minX = _a.minX, maxX = _a.maxX, minY = _a.minY, maxY = _a.maxY, xAxisLabel = _a.xAxisLabel, yAxisLabel = _a.yAxisLabel;
        //note: to support multiple 'scatter series' the data set has to be broken in corresponding discrete sets
        //for now, we're taking the first dataPointDef only
        var dataPointDef = graphDef.dataPointDefs[0];
        var xType = this.axisType(this.props.graphContext, dataPointDef.xAxisName, this.props.data);
        var yType = this.axisType(this.props.graphContext, dataPointDef.name, this.props.data);
        if (dataPointDef) {
            var legendKey = dataPointDef.legendKey || dataPointDef.name;
            if (graphDef.displayQuadrantLines) {
                var xRef = React.createElement(recharts_1.ReferenceLine, { x: (maxX + minX) / 2, stroke: "lightGrey", strokeWidth: "8" });
                var yRef = React.createElement(recharts_1.ReferenceLine, { y: (maxY + minY) / 2, stroke: "lightGrey", strokeWidth: "8" });
            }
            return (React.createElement("div", { className: "cv-graph-container" },
                React.createElement(recharts_1.ResponsiveContainer, { width: "99%", height: "95%" },
                    React.createElement(recharts_1.ScatterChart, { margin: { top: 20, right: 50, left: 20, bottom: 5 } },
                        React.createElement(recharts_1.XAxis, { dataKey: dataPointDef.xAxisName, name: dataPointDef.xAxisName, domain: [minX, maxX], label: xAxisLabel, ref: "xAxis" }),
                        React.createElement(recharts_1.YAxis, { dataKey: dataPointDef.name, name: legendKey, domain: [minY, maxY], label: yAxisLabel, ref: "yAxis" }),
                        xRef,
                        yRef,
                        (function () {
                            if (_this.props.bubble) {
                                if (dataPointDef.bubbleRadiusName) {
                                    return React.createElement(recharts_1.ZAxis, { dataKey: dataPointDef.bubbleRadiusName, name: dataPointDef.bubbleRadiusName, range: [10000, 50000] });
                                }
                                else {
                                    return React.createElement(recharts_1.ZAxis, { dataKey: dataPointDef.bubbleRadiusName, name: dataPointDef.bubbleRadiusName, range: [10000, 10000] });
                                }
                            }
                            else {
                                return React.createElement(recharts_1.ZAxis, null);
                            }
                        })(),
                        React.createElement(recharts_1.CartesianGrid, null),
                        React.createElement(recharts_1.Tooltip, { cursor: { strokeDasharray: '3 3' }, formatter: this._formatterFunction, labelFormatter: this._labelFormatterFunction }),
                        React.createElement(recharts_1.Scatter, { data: this.props.data, onClick: function (data) { _this.handleClick(data.id); } }, this.props.data.map(function (data, i2) {
                            var f = data.annoColor ? data.annoColor : _this.colorForDataPointDef(dataPointDef, 0);
                            if (dataPointDef.bubbleRadiusName) {
                                f = _this._applyAlpha(f, .4);
                            }
                            ;
                            return React.createElement(recharts_1.Cell, { key: "cell-" + i2, fill: f });
                        }))))));
        }
        else {
            return null;
        }
    },
    _labelFormatterFunction: function (a, b, c, d) {
        return "here";
    },
    _formatterFunction: function (a, b, c, d) {
        return "here";
    }
});
exports.CvPieChart = React.createClass({
    mixins: [exports.CvChart],
    render: function () {
        var _this = this;
        var graphContext = this.props.graphContext;
        var graphDef = graphContext.graphDef;
        //note: to support multiple 'pie levels' the data set has to be broken in corresponding discrete sets
        //for now, we're taking the first dataPointDef only
        var dataPointDef = graphDef.dataPointDefs[0];
        if (dataPointDef) {
            var propDef_1 = graphContext.propDefAtName(dataPointDef.name);
            var legendKey = dataPointDef.legendKey || dataPointDef.name;
            return (React.createElement("div", { className: "cv-graph-container" },
                React.createElement(recharts_1.ResponsiveContainer, { width: "99%", height: "95%" },
                    React.createElement(recharts_1.PieChart, { margin: { top: 20, right: 50, left: 20, bottom: 5 }, onClick: function (data) { _this.handleClick(data.id); } },
                        React.createElement(recharts_1.Tooltip, { cursor: { strokeDasharray: '3 3' } }),
                        React.createElement(recharts_1.Legend, null),
                        React.createElement(recharts_1.Pie, { data: this.props.data, nameKey: this.props.identPropName, valueKey: dataPointDef.name, label: function (v) { return catavolt_sdk_1.PropFormatter.formatValueForRead(v.value, propDef_1); } }, this.props.data.map(function (data, i) {
                            var color = data.annoColor ? data.annoColor : _this.colorForDataPointDef(dataPointDef, i);
                            return React.createElement(recharts_1.Cell, { key: "cell-" + i, fill: color });
                        }))))));
        }
        else {
            return null;
        }
    }
});
/*
 ***************************************************
 * Render a Graph
 ***************************************************
 */
exports.CvGraphPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentDidMount: function () {
    },
    getDefaultProps: function () {
        return {
            paneRef: null,
            formContext: null,
            graphContext: null,
            navigationListeners: [],
            selectionListener: null,
            stateChangeListeners: [],
            actionListeners: [],
            defaultSeriesColors: CV_GRAPH_DEFAULT_COLORS,
            navTarget: null,
            actionProvider: null
        };
    },
    getInitialState: function () {
        return {};
    },
    render: function () {
        var _this = this;
        var _a = this.state;
        var graphPaneProps = {
            paneRef: this.props.paneRef,
            formContext: this.props.formContext,
            queryContext: this.props.graphContext,
            stateChangeListeners: this.props.stateChangeListeners,
            actionListeners: this.props.actionListeners,
            actionProvider: this.props.actionProvider
        };
        return (React.createElement(catreact_1.CvQueryPane, __assign({}, graphPaneProps, { queryRenderer: function (cvContext, callback) {
                var graphContext = cvContext.scopeCtx.scopeObj;
                return React.createElement(exports.CvGraph, { graphContext: graphContext, lastRefreshTime: graphContext.lastRefreshTime, navigationListeners: _this.props.navigationListeners, selectionListener: _this.props.selectionListener, actionListeners: _this.props.actionListeners, stateChangeListeners: _this.props.stateChangeListeners, navTarget: _this.props.navTarget, defaultSeriesColors: _this.props.defaultSeriesColors });
            } })));
    },
});
exports.CvGraph = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentDidMount: function () {
    },
    getDefaultProps: function () {
        return {
            graphContext: null,
            lastRefreshTime: null,
            navigationListeners: [],
            selectionListener: null,
            stateChangeListeners: [],
            actionListeners: [],
            defaultSeriesColors: CV_GRAPH_DEFAULT_COLORS,
            navTarget: null,
        };
    },
    getInitialState: function () {
        return {};
    },
    render: function () {
        var _this = this;
        var graphContext = this.props.graphContext;
        var selectionAdapter = new catreact_1.CvValueAdapter();
        var selectionListener = selectionAdapter.createValueListener();
        return React.createElement(catreact_1.CvAction, { actionId: graphContext.graphDef.defaultActionId, paneContext: graphContext, navigationListeners: this.props.navigationListeners, actionListeners: this.props.actionListeners, stateChangeListeners: this.props.stateChangeListeners, selectionProvider: selectionAdapter, renderer: function (cvContext, callback) {
                var clickHandler = function (id) {
                    selectionListener([id]);
                    // Because the selectionListener is setting state that needs to be read by the callback,
                    // don't fire the callback until the state is set.  Accomplished by setting some dummy
                    // state.
                    _this.setState({}, function () { callback.fireAction(); });
                };
                var graphDef = graphContext.graphDef;
                var records = catavolt_sdk_1.ArrayUtil.copy(graphContext.scroller.buffer);
                var points = graphDef.graphType === catavolt_sdk_1.GraphDef.GRAPH_TYPE_PIE ?
                    _this._generateDataForPieChart(graphContext, records) : _this._generateData(graphContext, records);
                var identPropName = graphDef.identityDataPointDef.name;
                var plotTypes = graphDef.dataPointDefs.map(function (dataPointDef) { return dataPointDef.plotType; }).filter(function (v) { return !!v; });
                if (graphDef.graphType === catavolt_sdk_1.GraphDef.GRAPH_TYPE_CARTESIAN) {
                    if (plotTypes.length === 0 || plotTypes.indexOf(catavolt_sdk_1.GraphDef.PLOT_TYPE_BAR) > -1) {
                        return React.createElement(exports.CvBarChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, clickHandler: clickHandler });
                    }
                    else if (plotTypes.indexOf(catavolt_sdk_1.GraphDef.PLOT_TYPE_LINE) > -1) {
                        return React.createElement(exports.CvLineChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, clickHandler: clickHandler });
                    }
                    else if (plotTypes.indexOf(catavolt_sdk_1.GraphDef.PLOT_TYPE_STACKED) > -1) {
                        return React.createElement(exports.CvBarChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, stack: true, clickHandler: clickHandler });
                    }
                    else if (plotTypes.indexOf(catavolt_sdk_1.GraphDef.PLOT_TYPE_SCATTER) > -1) {
                        return React.createElement(exports.CvScatterChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, clickHandler: clickHandler });
                    }
                    else if (plotTypes.indexOf(catavolt_sdk_1.GraphDef.PLOT_TYPE_BUBBLE) > -1) {
                        return React.createElement(exports.CvScatterChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, bubble: true, clickHandler: clickHandler });
                    }
                    else {
                        return null;
                    }
                }
                else if (graphDef.graphType === catavolt_sdk_1.GraphDef.GRAPH_TYPE_PIE) {
                    return React.createElement(exports.CvPieChart, { graphContext: graphContext, data: points, identPropName: identPropName, defaultSeriesColors: _this.props.defaultSeriesColors, clickHandler: clickHandler });
                }
                else {
                    return null;
                }
            } });
    },
    shouldComponentUpdate: function (nextProps, nextState) {
        /*
         This is an 'inclusive' change list - updates DO NOT happen by default
         Desired updates should be explicitly added here
         */
        //update if we have new list data
        if (nextProps.lastRefreshTime) {
            if (this.props.lastRefreshTime) {
                if (nextProps.lastRefreshTime.getTime() > this.props.lastRefreshTime.getTime()) {
                    return true;
                }
            }
            else {
                return true;
            }
        }
        return false;
    },
    _formatProp: function (prop, propDef) {
        if (propDef && !propDef.isNumericType) {
            return catavolt_sdk_1.PropFormatter.formatValueForRead(prop.value, propDef);
        }
        else if (typeof prop.value !== 'number') {
            return catavolt_sdk_1.PropFormatter.formatValueForRead(prop.value, null);
        }
        else {
            return Number(prop.value);
        }
    },
    _generateData: function (graphContext, records) {
        var _this = this;
        var graphDef = graphContext.graphDef;
        //prop for 1 axis (or dimension)
        var identPropName = graphDef.identityDataPointDef.name;
        var points = records.map(function (record) {
            var item = {};
            item['id'] = record.objectId;
            item[identPropName] = _this._formatProp(record.propAtName(identPropName), graphContext.propDefAtName(identPropName));
            item['annoColor'] = record.foregroundColor;
            //possible props for other axes (or dimensions)
            graphDef.dataPointDefs.forEach(function (dataPointDef) {
                var propName = dataPointDef.name;
                if (propName)
                    item[propName] = _this._formatProp(record.propAtName(propName), graphContext.propDefAtName(propName));
                var xAxisName = dataPointDef.xAxisName;
                if (xAxisName)
                    item[xAxisName] = _this._formatProp(record.propAtName(xAxisName), graphContext.propDefAtName(xAxisName));
                var radiusName = dataPointDef.bubbleRadiusName;
                if (radiusName)
                    item[radiusName] = Number(record.propAtName(radiusName).value);
            });
            return item;
        });
        return points;
    },
    _generateDataForPieChart: function (graphContext, records) {
        var _this = this;
        /* The pie chart requires the primary series to be numeric */
        var graphDef = graphContext.graphDef;
        //prop for 1 axis (or dimension)
        var identPropName = graphDef.identityDataPointDef.name;
        var points = records.map(function (record) {
            var item = {};
            item['id'] = record.objectId;
            item[identPropName] = _this._formatProp(record.propAtName(identPropName), graphContext.propDefAtName(identPropName));
            item['annoColor'] = record.foregroundColor;
            graphDef.dataPointDefs.forEach(function (dataPointDef) {
                var propName = dataPointDef.name;
                //force numeric typing
                if (propName)
                    item[propName] = Number(record.propAtName(propName).value);
            });
            return item;
        });
        return points;
    },
});
