"use strict";
/**
 * Created by rburson on 3/15/16.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var catreact_1 = require("catreact");
var catavolt_sdk_1 = require("catavolt-sdk");
/*
 ***************************************************
 * Render a Workbench with Icons
 ***************************************************
 */
exports.CvGraphicalWorkbench = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentDidMount: function () {
        this.refresh(this.props);
    },
    componentWillReceiveProps: function (nextProps) {
        this.refresh(nextProps);
    },
    appWinDef: function () {
        return this.props.appWinDef || this.firstInScope(catavolt_sdk_1.AppWinDef);
    },
    getDefaultProps: function () {
        return {
            appWinDef: null,
            launchListeners: [],
            actionListeners: [],
            numCols: 3,
            selectionProvider: null,
            workbench: null,
            workbenchId: null
        };
    },
    getInitialState: function () {
        return { workbench: null };
    },
    getChildContext: function () {
        var ctx = this.getDefaultChildContext();
        ctx.cvContext.scopeCtx.scopeObj = this.workbench();
        return ctx;
    },
    refresh: function (props) {
        if (props.workbench) {
            this.setState({ workbench: props.workbench });
        }
        else if (props.workbenchId) {
            var workbench = this._workbenchForId(props.workbenchId);
            if (workbench) {
                this.setState({ workbench: workbench });
            }
            else {
                this.setState({ workbench: this.appWinDef().workbenches[0] });
            }
        }
        else {
            this.setState({ workbench: this.appWinDef().workbenches[0] });
        }
        var selectionProvider = props.selectionProvider;
        if (selectionProvider) {
            selectionProvider.subscribe(this._updateWorkbench);
        }
    },
    render: function () {
        var _this = this;
        var workbench = this.workbench();
        if (workbench) {
            return (React.createElement(catreact_1.CvWorkbench, { workbench: workbench, appWinDef: this.appWinDef(), renderer: function (cvContext) {
                    var cssCols = Math.floor(12 / _this.props.numCols);
                    var workbench = cvContext.scopeCtx.scopeObj;
                    var newChildren = [];
                    var rowElems = [];
                    workbench.workbenchLaunchActions.forEach(function (launchAction, i) {
                        rowElems.push(React.createElement(catreact_1.CvLauncher, { launchAction: launchAction, key: launchAction.actionId, launchListeners: _this.props.launchListeners, actionListeners: _this.props.actionListeners, renderer: function (cvContext, callback) {
                                var launcher = cvContext.scopeCtx.scopeObj;
                                return (React.createElement("div", { onClick: function () { callback.fireLaunchAction(); }, className: "col-sm-" + cssCols + " cv-launcher-container cv-target animated fadeIn", id: launcher.alias || launcher.actionId },
                                    React.createElement("img", { className: "cv-launcher-icon img-responsive center-block", src: launcher.iconBase, onError: _this._loadPlaceholder }),
                                    React.createElement("h4", { className: "cv-launcher-text small text-center" }, launcher.name)));
                            } }));
                        if (i % _this.props.numCols == _this.props.numCols - 1) {
                            newChildren.push(React.createElement('div', { className: 'row', key: i }, rowElems));
                            rowElems = [];
                        }
                    });
                    if (rowElems.length > 0) {
                        newChildren.push(React.createElement('div', { className: 'row', key: '00' }, rowElems));
                    }
                    return React.createElement("div", null, newChildren);
                } }));
        }
        else {
            return null;
        }
    },
    workbench: function () {
        return this.state.workbench;
    },
    _loadPlaceholder: function (e) {
        e.preventDefault();
        e.target.src = 'http://cdn.churchm.ag/wp-content/uploads/2013/06/404-Space-Invaders.png';
    },
    _updateWorkbench: function (workbench) {
        this.setState({ workbench: workbench });
    },
    _workbenchForId: function (id) {
        var appWinDef = this.appWinDef();
        var workbench = null;
        appWinDef.workbenches.some(function (wb) {
            if (wb.workbenchId == id) {
                workbench = wb;
                return true;
            }
            else {
                return false;
            }
        });
        return workbench;
    }
});
exports.CvGraphicalWorkbenchPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    appWinDef: function () {
        return this.props.appWinDef || this.firstInScope(catavolt_sdk_1.AppWinDef);
    },
    getDefaultProps: function () {
        return { appWinDef: null, workbench: null, workbenchId: null, workbenchRenderer: null };
    },
    render: function () {
        var workbench = this.workbench();
        if (this.props.renderer) {
            return this.props.renderer(this.getChildContext().cvContext);
        }
        else {
            return (React.createElement("div", { className: "cv-workbench-manager-container animated fadeIn" },
                React.createElement("div", { className: "cv-panel panel panel-primary cv-workbench-panel" },
                    React.createElement("div", { className: "cv-panel-heading panel-heading" },
                        React.createElement("h3", { className: "cv-panel-title panel-title" }, workbench ? workbench.name : '')),
                    React.createElement("div", { className: "cv-panel-body cv-workbench-panel-body" },
                        React.createElement("div", { className: "cv-workbench-scroll-container" }, this.props.workbenchRenderer(workbench))))));
        }
    },
    workbench: function () {
        if (this.props.workbenchId) {
            var workbench = this._workbenchForId(this.props.workbenchId);
            if (workbench) {
                return workbench;
            }
            else {
                return this.appWinDef().workbenches.length > 0 ? this.appWinDef().workbenches[0] : null;
            }
        }
    },
    _workbenchForId: function (id) {
        var appWinDef = this.appWinDef();
        var workbench = null;
        appWinDef.workbenches.some(function (wb) {
            if (wb.workbenchId == id) {
                workbench = wb;
                return true;
            }
            else {
                return false;
            }
        });
        return workbench;
    }
});
