"use strict";
/**
 * Created by rburson on 3/18/16.
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *  Warning!  This component required 'wrapping' a non-React style, traditional javascript grid component (ag-grid)
 *  The wrapped component is very stateful and requires interaction and control with a very 'imperative' javascript api,
 *  thus this component is not a good example the stateless, functional components that React normally promotes.
 *  There has been much work done here to 'work around' the issues created by the grid component and it's requirements.
 *  i.e. the grid component does not gracefully survive React component updates.
 *
 */
var React = require("react");
var ag_grid_react_1 = require("ag-grid-react");
var Clipboard = require("clipboard");
var catreact_1 = require("catreact");
var catreact_html_1 = require("./catreact-html");
var catavolt_sdk_1 = require("catavolt-sdk");
var CV_GRID_PAGE_SIZE = 200;
var CV_GRID_ROW_OVERFLOW_SIZE = 1;
var CV_GRID_ROW_HEIGHT = 25;
var CV_GRID_ROW_HEIGHT_IMAGES = 50;
var CvGridEventType;
(function (CvGridEventType) {
    CvGridEventType[CvGridEventType["SELECTION_EVENT"] = 0] = "SELECTION_EVENT";
    CvGridEventType[CvGridEventType["TOTAL_ROWS_LOADED_EVENT"] = 1] = "TOTAL_ROWS_LOADED_EVENT";
    CvGridEventType[CvGridEventType["FIRST_VISIBLE_ROW_CHANGE_EVENT"] = 2] = "FIRST_VISIBLE_ROW_CHANGE_EVENT";
})(CvGridEventType = exports.CvGridEventType || (exports.CvGridEventType = {}));
//the Grid control doesn't allow periods in column key names, so we switch back and forth
var CvGridPanel_DOT_REPL = '_d_o_t_';
/*
 ***************************************************
 * Render a ListContext
 ***************************************************
 */
exports.CvGridPanel = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentWillMount: function () {
        //set up unmanaged, init values
        var numInitialRows = this.props.numInitialRows ? this.props.numInitialRows : CV_GRID_PAGE_SIZE;
        this.setState({ numInitialRows: numInitialRows });
    },
    componentDidMount: function () {
        /* Allow for copying cell content */
        new Clipboard('.cv-clipboard-target', {
            text: function (trigger) {
                return trigger.innerText;
            }
        });
        /*
            register to receive events from the 'popup search' editor model
            the list pane will refresh itself when the search is submitted but we need
            to reset the data source before that happens
        */
        this.eventRegistry()
            .subscribe(this._dataChangeListener, catreact_1.CvEventType.STATE_CHANGE);
    },
    componentWillUnmount: function () {
        this.eventRegistry().unsubscribe(this._dataChangeListener);
    },
    componentWillReceiveProps: function (props, state) {
        //Log.debug('GridPanel will receive props:');
        //Log.debug(props);
    },
    componentWillUpdate: function (props, state) {
        //Log.debug('GridPanel will update: ');
        //Log.debug(props);
    },
    getDefaultProps: function () {
        return {
            paneRef: null,
            formContext: null,
            listContext: null,
            navigationListeners: [],
            actionListeners: [],
            gridListener: null,
            stateChangeListeners: [],
            actionProvider: null,
            searchable: false,
            initialSelectedItems: null,
            numInitialRows: CV_GRID_PAGE_SIZE,
            initialFirstVisibleRow: null,
            gridHeight: '100%'
        };
    },
    getInitialState: function () {
        return { searchNavEvent: null,
            quickFilterNavResult: null,
            complexSearchNavResult: null,
            numInitialRows: null,
            initialFirstVisibleRow: null
        };
    },
    render: function () {
        var _this = this;
        /* We can't do ES7 style rest destructuring in Typescript yet, so we do this... */
        var listPaneProps = {
            paneRef: this.props.paneRef,
            formContext: this.props.formContext,
            queryContext: this.props.listContext,
            stateChangeListeners: this.props.stateChangeListeners,
            actionListeners: this.props.actionListeners,
            actionProvider: this.props.actionProvider
        };
        return (React.createElement(catreact_1.CvListPane, __assign({}, listPaneProps, { recordPageSize: this.state.numInitialRows, queryRenderer: function (cvContext, callback) {
                var listContext = cvContext.scopeCtx.scopeObj;
                return (React.createElement("div", { className: "cv-component-container cv-grid-container" },
                    (function () {
                        return _this.props.searchable ?
                            React.createElement(catreact_1.CvAction, { actionId: '#search', fireOnLoad: function (success, error) { }, paneContext: listContext, navigationListeners: [function (event) {
                                        //hold on to the nav event so we can pass it along later to the complex search popup
                                        var newState = { complexSearchNavResult: event.eventObj, searchNavEvent: event };
                                        if (_this._unifiedSearchSupport()) {
                                            newState["quickFilterNavResult"] = event.eventObj;
                                        }
                                        _this.setState(newState);
                                    }] }) : null;
                    })(),
                    (function () {
                        //older server versions need to use the deprecated 'searchQuery' action for keyword search
                        return _this.props.searchable && !_this._unifiedSearchSupport() ?
                            React.createElement(catreact_1.CvAction, { actionId: 'searchQuery', fireOnLoad: function (success, error) { }, paneContext: listContext, navigationListeners: [function (event) {
                                        _this.setState({ quickFilterNavResult: event.eventObj });
                                    }] }) : null;
                    })(),
                    React.createElement("div", { className: "cv-component-toolbar" },
                        React.createElement(exports.CvQuickFilter, { filterValueListener: function (filterValue) { if (_this.table) {
                                _this.table.quickFilter(filterValue);
                            } }, ref: function (me) { return _this.quickFilter = me; } }),
                        React.createElement("button", { className: "btn btn-default", onClick: callback.refresh, title: "Refresh All" },
                            React.createElement("span", { className: "glyphicon glyphicon-refresh", "aria-hidden": "true" })),
                        (function () {
                            /*
                             The 'search' button fires a typical navigation which opens a search pane, which shows in a modal popup
                             We're passing along the previously obtained 'search navigation' result
                            */
                            return _this.props.searchable && _this.state.searchNavEvent ?
                                React.createElement(catreact_1.CvNavigation, { navigationResult: _this.state.searchNavEvent.eventObj },
                                    React.createElement(catreact_1.CvForm, { formContext: _this.state.searchNavEvent.eventObj.navRequest },
                                        React.createElement(catreact_1.CvSearchPane, { paneRef: 0, initMode: catreact_1.CvInitMode.MANUAL, detailsRenderer: function (cvContext, entityRec, searchCallback) {
                                                return React.createElement("button", { className: "btn btn-default", onClick: function () {
                                                        searchCallback.reopenSearch(function (success, error) {
                                                            if (error) {
                                                                _this.props.eventRegistry.publishError('complex search reopen failed with '
                                                                    + catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                                                            }
                                                            else {
                                                                if (_this.quickFilter.isSet())
                                                                    _this.quickFilter.resetText();
                                                                _this.props.navigationListeners.forEach(function (listener) { return listener(_this.state.searchNavEvent); });
                                                            }
                                                        });
                                                    }, title: "Search" },
                                                    React.createElement("span", { className: "glyphicon glyphicon-search", "aria-hidden": "true" }));
                                            } })))
                                : null;
                        })()),
                    React.createElement("div", { className: "cv-component-body" }, (function () {
                        /* make sure that the search actions have been opened */
                        return _this._areSearchHandlesReady() ?
                            React.createElement(catreact_1.CvNavigation, { navigationResult: _this.state.quickFilterNavResult },
                                React.createElement(catreact_1.CvForm, { formContext: _this.state.quickFilterNavResult.navRequest },
                                    React.createElement(catreact_1.CvSearchPane, { paneRef: 0, initMode: catreact_1.CvInitMode.MANUAL, detailsRenderer: function (cvContext, entityRec, quickFilterCallback) {
                                            return React.createElement(exports.CvGridTable, { listContext: listContext, lastRefreshTime: listContext.lastRefreshTime, initialSelectedItems: _this.props.initialSelectedItems, numInitialRows: _this.props.numInitialRows, gridHeight: _this.props.gridHeight, initialFirstVisibleRow: _this.props.initialFirstVisibleRow, queryPaneCallback: callback, searchNavResult: _this.state.searchNavEvent.eventObj, quickFilterCallback: quickFilterCallback, eventRegistry: _this.eventRegistry(), navigationListeners: _this.props.navigationListeners, actionListeners: _this.props.actionListeners, gridListener: _this.props.gridListener, stateChangeListeners: _this.props.stateChangeListeners, ref: function (table) { return _this.table = table; } });
                                        } }))) : null;
                    })())));
            } })));
    },
    shouldComponentUpdate: function (nextProps, nextState) {
        /*
         NOTE:
         initialSelectedItems, numInitialRows, initialFirstVisibleRow are all unmanaged properties, meaning
         that when they change, they are not synchronized with component state (we ignore them)
         Here they are simply changing URL params to be retained only for navigation or a refresh
         */
        /*
            This is an 'exclusive' change list - updates happen by default unless included here
            Props changes that should not fire updates, should be added here
         */
        //Don't update for selection changes as this is NOT a managed param - the Grid control handles selections internally
        //determine if this was a selection change
        //if so avoid a component update
        var initialSelectedItems = this.props.initialSelectedItems;
        var nextSelectedItems = nextProps.initialSelectedItems;
        //do we already have selected items
        //if so have we simply added or removed a selection
        if (initialSelectedItems != null) {
            if (nextSelectedItems && (initialSelectedItems.length != nextSelectedItems.length)) {
                return false;
            }
        }
        else {
            //we don't have a selection but now, but having an incoming selection
            if (nextSelectedItems != null) {
                return false;
            }
        }
        //Don't update for changes in original num rows - also handled internally by Grid control
        //this is NOT a managed parameter
        if (nextProps.numInitialRows != this.props.numInitialRows) {
            return false;
        }
        //Don't update for changes in initialFirstVisibleRow
        //this is NOT a managed parameter
        if (nextProps.initialFirstVisibleRow != this.props.initialFirstVisibleRow) {
            return false;
        }
        //otherwise we'll continue to update the grid...
        return true;
    },
    _dataChangeListener: function (dataChangeResult) {
        var eventSource = dataChangeResult.eventObj.source;
        var eventType = dataChangeResult.eventObj.type;
        var complexSearchNavResult = this.state.complexSearchNavResult;
        //listen for the modal search submission
        if (eventType === catreact_1.CvStateChangeType.DATA_CHANGE) {
            if (eventSource && complexSearchNavResult && complexSearchNavResult.navRequest) {
                if (eventSource.parentContext == complexSearchNavResult.navRequest) {
                    this.table.resetDataSource();
                }
            }
        }
    },
    _areSearchHandlesReady: function () {
        return this.state.quickFilterNavResult && this.state.searchNavEvent;
    },
    _unifiedSearchSupport: function () {
        return this.catavolt().isFeatureSetAvailable("Unified_Search");
    }
});
exports.CvQuickFilter = React.createClass({
    getDefaultProps: function () {
        return { filterValueListener: null };
    },
    getInitialState: function () {
        return { searchText: "", isSet: false };
    },
    isSet: function () {
        return this.state.searchText;
    },
    render: function () {
        return React.createElement("span", { className: "form-inline cv-quick-filter" },
            React.createElement("input", { placeholder: "Quick Filter...", className: "form-control", type: "text", value: this.state.searchText, onChange: this._onFilterTextChange, onKeyUp: this._filterKeyUp }),
            React.createElement("button", { className: "btn btn-default", onClick: this.resetText, title: "Reset Quick Filter" },
                React.createElement("span", { className: "glyphicon glyphicon-remove-sign", "aria-hidden": "true" })));
    },
    resetText: function () {
        this.setState({ searchText: "" });
        var listener = this.props.filterValueListener;
        if (listener) {
            listener("");
        }
    },
    _filterKeyUp: function (e) {
        if (e.keyCode === 13) {
            var listener = this.props.filterValueListener;
            if (listener) {
                listener(this.state.searchText);
            }
        }
    },
    _onFilterTextChange: function (e) {
        var value = e.target.value ? e.target.value : "";
        this.setState({ searchText: value });
    }
});
exports.CvGridTable = React.createClass({
    autoSize: function () {
        var columns = this.columnApi.getAllColumns().filter(function (col) { return col.visible; }).map(function (col) { return col.colDef.field; });
        this.columnApi.autoSizeColumns(columns);
    },
    autoFit: function () {
        this.gridApi.sizeColumnsToFit();
    },
    componentWillMount: function () {
        //set up unmanaged, init values
        var numInitialRows = this.props.numInitialRows ? this.props.numInitialRows : CV_GRID_PAGE_SIZE;
        this.setState({ numInitialRows: numInitialRows });
        var initialFirstVisibleRow = this.props.initialFirstVisibleRow ? this.props.initialFirstVisibleRow : 0;
        this.setState({ initialFirstVisibleRow: initialFirstVisibleRow });
        this.setState({ initialSelectedItems: this.props.initialSelectedItems });
    },
    componentDidMount: function () {
    },
    componentWillReceiveProps: function (props, state) {
        //Log.debug('GridTable will receive props:');
        //Log.debug(props);
    },
    componentWillUpdate: function (props, state) {
        //Log.debug('GridTable will update:');
        //Log.debug(props);
    },
    componentDidUpdate: function (prevProps, prevState) {
        //Log.debug('GridTable component did update');
        //make sure there's not another update in progress, as this breaks the gridApi call here
        if (this.gridApi && !this.props.queryPaneCallback.isInProgress()) {
            //Log.debug('calling refreshInfintePageCache');
            this.gridApi.refreshInfinitePageCache();
        }
    },
    getDefaultProps: function () {
        return {
            listContext: null,
            lastRefreshTime: null,
            initialSelectedItems: null,
            numInitialRows: CV_GRID_PAGE_SIZE,
            gridHeight: '100%',
            initialFirstVisibleRow: null,
            queryPaneCallback: null,
            searchNavResult: null,
            quickFilterCallback: null,
            eventRegistry: null,
            navigationListeners: [],
            actionListeners: [],
            gridListener: null,
            stateChangeListeners: []
        };
    },
    getInitialState: function () {
        return { numInitialRows: null, initialFirstVisibleRow: null, initialSelectedItems: null, forceAutoSize: false };
    },
    quickFilter: function (filterText) {
        var _this = this;
        //submit the quick search
        var quickFilterCallback = this.props.quickFilterCallback;
        var queryPaneCallback = this.props.queryPaneCallback;
        if (quickFilterCallback) {
            quickFilterCallback.reopenSearch(function (success, error) {
                if (error) {
                    _this.props.eventRegistry.publishError('quickFilter reopen failed with ' + catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                }
                else {
                    quickFilterCallback.clearSearchValues();
                    quickFilterCallback.setPropValue('keyword', filterText);
                    //this is key - ag-grid must rebuild the grid instance
                    _this.resetDataSource();
                    //this.setState({forceAutoSize:true});
                    quickFilterCallback.submitSearch(function (success, error) {
                        if (error) {
                            queryPaneCallback.refresh();
                            _this.props.eventRegistry.publishError('quickFilter error ' + catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                        }
                    });
                }
            });
        }
    },
    render: function () {
        var _this = this;
        var _a = this.props, listContext = _a.listContext, queryPaneCallback = _a.queryPaneCallback, searchNavResult = _a.searchNavResult, navigationListeners = _a.navigationListeners, actionListeners = _a.actionListeners, stateChangeListeners = _a.stateChangeListeners, eventRegistry = _a.eventRegistry;
        var entityRecs = catavolt_sdk_1.ArrayUtil.copy(listContext.scroller.buffer);
        var hasBinaries = this._anyBinaries(listContext, entityRecs);
        var rowHeight = hasBinaries ? CV_GRID_ROW_HEIGHT_IMAGES : CV_GRID_ROW_HEIGHT;
        return (React.createElement(catreact_1.CvNavigation, { navigationResult: searchNavResult },
            React.createElement(catreact_1.CvForm, { formContext: searchNavResult.navRequest },
                React.createElement(catreact_1.CvSearchPane, { paneRef: 0, initMode: catreact_1.CvInitMode.MANUAL, detailsRenderer: function (cvContext, entityRec, searchCallback) {
                        var columnDefs = listContext.listDef.activeColumnDefs.map(function (columnDef) {
                            //@TODO - fully integrate custom filtering
                            //filter: this._getFilterType(columnDef.propertyDef),
                            //filterParams: {apply:true},
                            //the grid doesn't allow '.' in key names
                            return {
                                headerName: columnDef.heading,
                                field: columnDef.name.replace('.', CvGridPanel_DOT_REPL),
                                width: _this._getColumnWidth(columnDef),
                                cellRendererFramework: exports.CvGridTableCell
                            };
                        });
                        columnDefs[0].pinned = 'left';
                        columnDefs[0].cellRendererFramework = exports.CvGridTableCellColumn0;
                        if (!_this.dataSource) {
                            _this.dataSource = new CvGridDataSource(-1, listContext, searchCallback, queryPaneCallback, navigationListeners, actionListeners, stateChangeListeners, eventRegistry);
                        }
                        else {
                            _this.dataSource.update(listContext, searchCallback, queryPaneCallback, navigationListeners, actionListeners, stateChangeListeners, eventRegistry);
                        }
                        return (React.createElement("div", { style: { height: _this.props.gridHeight }, className: "ag-fresh" },
                            React.createElement(ag_grid_react_1.AgGridReact, { onGridReady: _this._onGridReady, showToolPanel: false, onRowSelected: _this._onRowSelected, onRowDoubleClicked: _this._onRowDoubleClicked, onBodyScroll: catreact_1.UIUtil.debounce(_this._onBodyScroll.bind(_this, rowHeight), 200), getRowStyle: _this._getRowStyle, getRowNodeId: _this._getRowNodeId, rowHeight: rowHeight, columnDefs: columnDefs, enableServerSideSorting: true, enableServerSideFilter: false, suppressRowClickSelection: "false", rowSelection: "multiple", rowDeselection: true, enableColResize: true, enableSorting: false, enableFilter: false, groupHeaders: false, suppressCellSelection: true, debug: false, rowModelType: 'infinite', infiniteInitialRowCount: _this.state.numInitialRows, paginationPageSize: _this.state.numInitialRows, infiniteBlockSize: _this.state.numInitialRows, maxConcurrentDatasourceRequests: 1, sortingOrder: ['asc', 'desc'], datasource: _this.dataSource, paginationOverflowSize: CV_GRID_ROW_OVERFLOW_SIZE })));
                    } }))));
    },
    resetDataSource: function () {
        this.dataSource = null;
    },
    shouldComponentUpdate: function (nextProps, nextState) {
        /*
         NOTE:
         initialSelectedItems, numInitialRows, initialFirstVisibleRow are all unmanaged properties, meaning
         that when they change, they are not synchronized with component state (we ignore them)
         They are simply changing URL params to be retained only for navigation or a refresh
         */
        //don't update if we've change the forceAutoSize value
        if (this.state.forceAutoSize != this.state.forceAutoSize) {
            return false;
        }
        //update only if we have new list data
        if (this.props.queryPaneCallback && !this.props.queryPaneCallback.isInProgress()) {
            if (nextProps.lastRefreshTime) {
                if (this.props.lastRefreshTime) {
                    if (nextProps.lastRefreshTime.getTime() > this.props.lastRefreshTime.getTime()) {
                        return true;
                    }
                }
                else {
                    return true;
                }
            }
        }
        return false;
    },
    _anyBinaries: function (listContext, entityRecs) {
        return listContext.entityRecDef.propertyDefs.some(function (propDef) { return propDef.isBinaryType; }) ||
            entityRecs.some(function (entityRec) { return !!entityRec.imageName || entityRec.props.some(function (prop) { return !!prop.imageName; }); });
    },
    _onBodyScroll: function (rowHeight, event) {
        /*
            1) Report change in 'first visible row' (unmanaged parameter)
         */
        //seems to be the only option for approximating the first visible row.  really need a better solution
        var vpr = this.gridApi.getVerticalPixelRange();
        var rowNum = Math.round(vpr.bottom / rowHeight) - 1;
        if (this.props.gridListener)
            this.props.gridListener({ type: CvGridEventType.FIRST_VISIBLE_ROW_CHANGE_EVENT, eventObj: rowNum });
    },
    _onGridReady: function (params) {
        this.gridApi = params.api;
        this.columnApi = params.columnApi;
        //relying on this event to tell us when more pages have been loaded and add to the scroll buffer
        this.gridApi.addEventListener('paginationChanged', this._onPaginationChanged);
        //relying on this event firing when brand new data is inserted
        this.gridApi.addEventListener('columnEverythingChanged', this._onColumnEverythingChanged);
        //relying on this event firing after rows are loaded and inserted (this seems to be the only indication)
        this.gridApi.addEventListener('scrollVisibilityChanged', this._onRowsInitialized);
        //uncomment this to log all the ag-grid events to the console for debugging
        this.gridApi.addGlobalListener(function (event, p1, p2, p3) { var e = event; catavolt_sdk_1.Log.debug(e); });
    },
    _onPaginationChanged: function (event) {
        /*
         *  1) Report change in total rows loaded (unmanaged param)
         */
        var model = this.gridApi.getModel();
        if (model) {
            var rowsLoaded = this.props.listContext.scroller.buffer.length;
            //don't update if rowsLoaded hasn't actually changed (i.e. on the first load)
            if (this.state.numInitialRows != rowsLoaded) {
                if (this.props.gridListener && (rowsLoaded > CV_GRID_PAGE_SIZE)) {
                    this.props.gridListener({ type: CvGridEventType.TOTAL_ROWS_LOADED_EVENT, eventObj: rowsLoaded });
                }
            }
        }
    },
    _onColumnEverythingChanged: function (event) {
        //if(this.state.forceAutoSize) {
        //The grid doesn't report an event when all rows are rendered, so we're
        //left with the horrible option of trying to time the autoSize.  somebody help.
        setTimeout(this.autoSize, 500);
        //this.setState({forceAutoSize:false});
        //}
    },
    _onRowDoubleClicked: function (event) {
        event.node.data.rowCallback.fireAction();
    },
    _onRowSelected: function (event) {
        /*
            1) Report change in selections (unmanaged parameter)
         */
        //get the oids of the selected rows and notify the gridListeners fo the selection
        if (event.node.data && event.node.data.entityRec) {
            var selectedItems = this.gridApi.getSelectedRows().map(function (data) { return data.entityRec.objectId; });
            if (this.props.gridListener)
                this.props.gridListener({ type: CvGridEventType.SELECTION_EVENT, eventObj: selectedItems });
        }
    },
    _onRowsInitialized: function (event) {
        var _this = this;
        //set the first visible row if specified
        if (this.state.initialFirstVisibleRow) {
            this.gridApi.ensureIndexVisible(this.state.initialFirstVisibleRow);
        }
        //select initial rows based on the supplied selectedItems oids
        var selectedItems = this.state.initialSelectedItems;
        if (selectedItems && selectedItems.length > 0) {
            selectedItems.forEach(function (selectedItem) {
                _this.gridApi.forEachNode(function (node) {
                    if (node.data && node.data.entityRec &&
                        (node.data.entityRec.objectId === selectedItem)) {
                        node.setSelected(true);
                    }
                });
            });
        }
        //autosize the columns
        this.autoSize();
        //remove this listener as we only want this to fire on the first load
        this.gridApi.removeEventListener('scrollVisibilityChanged', this._onRowsInitialized);
    },
    _getFilterType: function (propDef) {
        if (propDef.isDateType || propDef.isDateTimeType) {
            return 'date';
        }
        else if (propDef.isNumericType) {
            return 'number';
        }
        else {
            return 'text';
        }
    },
    _getRowStyle: function (params) {
        if (params.data && params.data.entityRec) {
            var styleInfo = catreact_html_1.CvDataAnno.generateStyleInfo(params.data.entityRec);
            return styleInfo.cssStyle;
        }
        else {
            return null;
        }
    },
    _getRowNodeId: function (data) {
        return data.entityRec.objectId;
    },
    _getColumnWidth: function (columnDef) {
        return columnDef.propertyDef.presLength < 200 || columnDef.propertyDef.presLength > 2000 ? 250 : columnDef.propertyDef.presLength;
    },
});
var CvGridDataSource = (function () {
    function CvGridDataSource(rowCount, listContext, searchCallback, queryPaneCallback, navigationListeners, actionListeners, stateChangeListeners, eventRegistry) {
        if (navigationListeners === void 0) { navigationListeners = []; }
        if (actionListeners === void 0) { actionListeners = []; }
        if (stateChangeListeners === void 0) { stateChangeListeners = []; }
        this.rowCount = rowCount;
        this.listContext = listContext;
        this.searchCallback = searchCallback;
        this.queryPaneCallback = queryPaneCallback;
        this.navigationListeners = navigationListeners;
        this.actionListeners = actionListeners;
        this.stateChangeListeners = stateChangeListeners;
        this.eventRegistry = eventRegistry;
        this.inProgress = false;
        this.init();
    }
    CvGridDataSource.prototype.update = function (listContext, searchCallback, queryPaneCallback, navigationListeners, actionListeners, stateChangeListeners, eventRegistry) {
        if (navigationListeners === void 0) { navigationListeners = []; }
        if (actionListeners === void 0) { actionListeners = []; }
        if (stateChangeListeners === void 0) { stateChangeListeners = []; }
        this.listContext = listContext;
        this.searchCallback = searchCallback;
        this.queryPaneCallback = queryPaneCallback;
        this.navigationListeners = navigationListeners;
        this.actionListeners = actionListeners;
        this.stateChangeListeners = stateChangeListeners;
        this.eventRegistry = eventRegistry;
        this.init();
    };
    CvGridDataSource.prototype.init = function () {
        this.searchCallback.suppressEvents(true);
        this.queryPaneCallback.suppressEvents(true);
    };
    CvGridDataSource.prototype.getRows = function (params) {
        var _this = this;
        var sortModel = params.sortModel;
        if (sortModel && sortModel.length > 0) {
            this._handleSort(sortModel, this.searchCallback, function (success, error) {
                if (error) {
                    _this.eventRegistry.publishError('CvGridPanel._handleSoft failed with ' + catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                }
                else if (success) {
                    //refresh the list
                    _this.listContext.scroller.refresh().onComplete(function (entityRecTry) {
                        if (entityRecTry.isFailure) {
                            _this.eventRegistry.publishError('CvGridPanel::getRows queryContext.scroller.refresh failed with ' + catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                        }
                        else {
                            _this._getRows(params);
                        }
                    });
                }
                else {
                    _this._getRows(params);
                }
            });
        }
        else {
            this._getRows(params);
        }
    };
    CvGridDataSource.prototype._getRows = function (params) {
        var _this = this;
        var startRow = params.startRow, endRow = params.endRow, successCallback = params.successCallback, failCallback = params.failCallback;
        var rowData = [];
        var currentRows = this.listContext.scroller.buffer;
        //any rows loaded yet?
        if (currentRows.length > 0) {
            this.pageForwardToRange(function (success, error) {
                if (error) {
                    _this.eventRegistry.publishError(catavolt_sdk_1.ObjUtil.formatRecAttr(error));
                    failCallback();
                }
                else {
                    var currentRows_1 = _this.listContext.scroller.buffer;
                    var lastIndex = -1;
                    lastIndex = (currentRows_1.length > endRow || _this.queryPaneCallback.hasMoreForward()) ? -1 :
                        (endRow > currentRows_1.length) ? currentRows_1.length : endRow;
                    rowData = _this._buildRows(currentRows_1.slice(startRow, endRow));
                    successCallback(rowData, lastIndex);
                }
            }, this.queryPaneCallback, startRow, endRow);
        }
        else {
            successCallback(rowData, 0);
        }
    };
    /*
     @TODO - fully integrate custom filtering!
     Resolve to true if the filter value has changed, otherwise false.
     private _handleFilters(filterModel:any, searchCallback:CvSearchPaneCallback,
     resultCallback:CvResultCallback<boolean>) {
     Object.keys(filterModel).forEach((key:string)=>{
     const colName = key;
     const pFilterValue = filterModel[key].filter;
     const pOperator = filterModel[key].type;
     });
     }
     */
    /*
     Resolve to true if the sort value has changed, otherwise false.
     */
    CvGridDataSource.prototype._handleSort = function (sortModel, searchCallback, resultCallback) {
        var colName = sortModel[0].colId.replace(CvGridPanel_DOT_REPL, '.');
        var isDesc = sortModel[0] && sortModel[0].sort.indexOf('d') === 0;
        var sortDirection = null;
        if ((isDesc && !searchCallback.isDescending(colName))) {
            sortDirection = 'DSC';
        }
        else if (!isDesc && !searchCallback.isAscending(colName)) {
            sortDirection = 'ASC';
        }
        if (sortDirection) {
            searchCallback.reopenSearch(function (success, error) {
                if (error) {
                    resultCallback(false, error);
                }
                else {
                    searchCallback.clearSortValues();
                    searchCallback.setSortValue(colName, sortDirection, 0);
                    searchCallback.submitSearch(function (success, error) {
                        resultCallback(!error, error);
                    });
                }
            });
        }
        else {
            resultCallback(false);
        }
    };
    CvGridDataSource.prototype._buildRows = function (newPage) {
        var _this = this;
        var entityRecs = catavolt_sdk_1.ArrayUtil.copy(newPage);
        return entityRecs.map(function (record) {
            var row = { entityRec: record };
            _this.listContext.listDef.activeColumnDefs.forEach(function (colDef) {
                var prop = record.propAtName(colDef.name);
                var styleInfo = catreact_html_1.CvDataAnno.generateStyleInfo(prop);
                var cellStyle = 'cv-grid-cell cv-target';
                //select "this record" so that the action can find the target via the selectionProvider
                var selectionAdapter = new catreact_1.CvValueAdapter();
                selectionAdapter.createValueListener()([record.objectId]);
                //the grid doesn't allow '.' in key names
                row[colDef.name.replace('.', CvGridPanel_DOT_REPL)] =
                    React.createElement(catreact_1.CvAction, { actionId: _this.listContext.listDef.defaultActionId, paneContext: _this.listContext, navigationListeners: _this.navigationListeners, actionListeners: _this.actionListeners, stateChangeListeners: _this.stateChangeListeners, selectionProvider: selectionAdapter, key: prop.name + '_', renderer: function (cvContext, callback) {
                            row['rowCallback'] = callback;
                            return (React.createElement("div", { className: "cv-grid-cell-wrapper" },
                                React.createElement(catreact_html_1.CvDataAnno, { dataAnnoStyle: styleInfo, paneContext: _this.listContext, wrapperElem: "div", wrapperElemProps: { className: cellStyle } },
                                    React.createElement(catreact_html_1.CvHtmlProp, { propName: prop.name, entityRec: record, paneContext: _this.listContext, imageClassName: "cv-image-grid-item-max-size", overrideValue: styleInfo.overrideText, dataPropNames: ['title'], wrapperElemProps: { className: 'cv-clipboard-target' } }))));
                        } });
            });
            return row;
        });
    };
    CvGridDataSource.prototype.pageForwardToRange = function (resultCallback, queryPaneCallback, startRow, endRow) {
        var _this = this;
        var currentRows = this.listContext.scroller.buffer;
        if (startRow >= currentRows.length || endRow > currentRows.length) {
            if (queryPaneCallback.hasMoreForward()) {
                queryPaneCallback.pageForward(function (num, error) {
                    var currentRows = _this.listContext.scroller.buffer;
                    if (error) {
                        resultCallback(false, error);
                    }
                    else if (startRow >= currentRows.length) {
                        //still haven't reached start row, try to page again
                        _this.pageForwardToRange(resultCallback, queryPaneCallback, startRow, endRow);
                    }
                    else {
                        //we've got the startRow, check the endRow
                        if (endRow > currentRows.length) {
                            //still haven't reached the endRow, try to page again
                            _this.pageForwardToRange(resultCallback, queryPaneCallback, startRow, endRow);
                        }
                        else {
                            resultCallback(true);
                        }
                    }
                });
            }
            else {
                if (startRow >= currentRows.length) {
                    resultCallback(false, new Error('Requested startRow is greater than total number of rows'));
                }
                else {
                    //no more pages, we've got the start row but no the end row
                    resultCallback(true);
                }
            }
        }
        else {
            //already loaded
            resultCallback(true);
        }
    };
    return CvGridDataSource;
}());
exports.CvGridTableCellColumn0 = React.createClass({
    render: function () {
        if (this.props.data) {
            return this.props.value || null;
        }
        else {
            return React.createElement("i", { className: "fa fa-spinner fa-pulse cv-table-spinner-size" });
        }
    }
});
exports.CvGridTableCell = React.createClass({
    render: function () {
        return this.props.value || null;
    }
});
