"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var catreact_1 = require("catreact");
exports.CvImageDefaultAddButtonClass = "cv-image-control-btn-add";
exports.CvImageDefaultRemoveButtonClass = "cv-image-control-btn-remove";
exports.CvImageDefaultUndoButtonClass = "cv-image-control-btn-undo";
exports.CvImageDefaultSignatureButtonClass = "cv-image-control-btn-signature";
/*
 ***************************************************
 * Render an image preserving aspect ratio
 ***************************************************
 */
exports.CvImage = React.createClass({
    mixins: [catreact_1.CvBaseMixin],
    componentDidMount: function () {
        if (this.props.hasFocus) {
            if (this.props.actions && this.props.actions.length == 1) {
                // If auto focus is requested and there is exactly one action, fire it.
                this.props.actions[0].callback();
            }
        }
    },
    componentWillReceiveProps: function (nextProps) {
        if (this.props.src != nextProps.src) {
            // No image, size is not known (may be removed and re-added)
            this.setState({ imageNaturalWidth: null, imageNaturalHeight: null });
        }
    },
    render: function () {
        var outterDivStyle = this.props.style ? this.props.style : {};
        var src = this.props.src;
        var knownImageSize = this._knownImageSize();
        var imageActions = this.props.actions ? this.props.actions : new Array();
        // If there is an explict width and height, we will nest this image into a div so we can
        // preserve the aspect ratio.
        if (outterDivStyle["width"] && outterDivStyle["height"]) {
            var sw = outterDivStyle["width"];
            var sh = outterDivStyle["height"];
            var managingFit = true;
        }
        else {
            managingFit = false;
        }
        var imageTop = 0;
        var imageLeft = 0;
        if (managingFit && knownImageSize) {
            // Alter the width and height styles to preserve the aspect ratio
            var fit = catreact_1.ImageUtil.calculateAspectRatioFit(this.state.imageNaturalWidth, this.state.imageNaturalHeight, sw, sh);
            if (fit.width < sw) {
                imageLeft = (sw - fit.width) / 2;
            }
            else if (fit.height < sh) {
                imageTop = (sh - fit.height) / 2;
            }
        }
        var className = "cv-image-controls-wrapper";
        if (this.props.className) {
            className = className + " " + this.props.className;
        }
        ;
        if (src) {
            if (managingFit) {
                // If managing the fit, it requires overriding the maxWidth/maxHeight values that might have been
                // specified to control image sizes.
                if (!fit) {
                    fit = { width: sw, height: sh };
                }
                ;
                var imageStyle = { position: 'absolute', width: fit.width, height: fit.height, top: imageTop, left: imageLeft,
                    maxWidth: 10000000, maxHeight: 10000000, pointerEvents: "none" };
                var imageCtl = React.createElement("img", { className: this.props.imageClassName, style: imageStyle, src: src, onLoad: this._imageLoaded });
            }
            else {
                imageCtl = React.createElement("img", { className: this.props.imageClassName, src: src, style: { pointerEvents: 'none' } });
            }
        }
        else {
            this.knownImageSize = null;
            className = className + " cv-no-image";
            if (this.props.className) {
                className = className + " " + this.props.className;
            }
            ;
        }
        if (this.props.readOnly != null) {
            this.props.readOnly ? className = className + " cv-readOnly" : className = className + " cv-maintainable";
        }
        return React.createElement("div", { style: outterDivStyle, className: className },
            React.createElement("div", { className: "cv-image-controls" }, imageActions.map(function (a, i) {
                return React.createElement("div", { className: "cv-image-control-btn", onClick: a.callback, key: i },
                    React.createElement("div", { className: a.buttonClassName }));
            })),
            imageCtl);
    },
    _imageLoaded: function (e) {
        if (!this._knownImageSize()) {
            this.setState({ imageNaturalWidth: e.target.naturalWidth, imageNaturalHeight: e.target.naturalHeight });
        }
    },
    _knownImageSize: function () {
        return this.state && this.state.imageNaturalHeight;
    }
});
