"use strict";
/**
 * Created by rburson on 12/28/16.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var catavolt_sdk_1 = require("catavolt-sdk");
var CvDisplayProperties = (function () {
    function CvDisplayProperties(properties) {
        this._properties = properties ? properties : {};
    }
    CvDisplayProperties.prototype.getAndUpdate = function (name, value) {
        var newDisplayProps = catavolt_sdk_1.ObjUtil.addAllProps(this._properties, {});
        newDisplayProps[name] = value;
        return new CvDisplayProperties(newDisplayProps);
    };
    CvDisplayProperties.prototype.getProperty = function (name) {
        return this._properties[name];
    };
    CvDisplayProperties.prototype.getAndRemove = function (name) {
        var newDisplayProps = catavolt_sdk_1.ObjUtil.addAllProps(this._properties, {});
        delete newDisplayProps[name];
        return new CvDisplayProperties(newDisplayProps);
    };
    CvDisplayProperties.prototype.serializeProperties = function () {
        var params = [];
        for (var p in this._properties)
            if (this._properties.hasOwnProperty(p)) {
                params.push(p + CvDisplayProperties.VALUE_DELIM + this.encodePropValue(this._properties[p]));
            }
        return params.join(CvDisplayProperties.PARAM_DELIM);
    };
    CvDisplayProperties.prototype.publishToListeners = function (displayPropChangeListeners) {
        var _this = this;
        displayPropChangeListeners.forEach(function (listener) { listener(_this); });
    };
    CvDisplayProperties.deserializeProperties = function (propString) {
        var params = propString.split(CvDisplayProperties.PARAM_DELIM);
        var properties = {};
        params.forEach(function (paramValues) {
            var values = paramValues.split(CvDisplayProperties.VALUE_DELIM);
            if (values.length > 0) {
                if (values.length > 1) {
                    properties[values[0]] = CvDisplayProperties.decodePropValue(values[1]);
                }
                else {
                    properties[values[0]] = null;
                }
            }
        });
        return new CvDisplayProperties(properties);
    };
    /*
        Note: we use Base64 to encode property value and any reserved chars we are using for delimiters
     */
    CvDisplayProperties.prototype.encodePropValue = function (propValue) {
        if (propValue == null || propValue == undefined) {
            return '';
        }
        else {
            return Array.isArray(propValue) ?
                CvDisplayProperties.ARRAY_ITEM_DELIM + propValue.map(function (v) { return catavolt_sdk_1.Base64.encode(String(v)); }).join(CvDisplayProperties.ARRAY_ITEM_DELIM)
                : catavolt_sdk_1.Base64.encode(String(propValue));
        }
    };
    CvDisplayProperties.decodePropValue = function (propValue) {
        if (propValue == null || propValue == undefined || propValue == '') {
            return null;
        }
        else {
            if (propValue.substr(0, 1) === ',') {
                if (propValue.length > 1) {
                    return propValue.substr(1).split(CvDisplayProperties.ARRAY_ITEM_DELIM).map(function (v) { return catavolt_sdk_1.Base64.decode(v); });
                }
                else {
                    return [];
                }
            }
            else {
                return catavolt_sdk_1.Base64.decode(propValue);
            }
        }
    };
    return CvDisplayProperties;
}());
CvDisplayProperties.PARAM_DELIM = '~';
CvDisplayProperties.VALUE_DELIM = ':';
CvDisplayProperties.ARRAY_ITEM_DELIM = ',';
exports.CvDisplayProperties = CvDisplayProperties;
