"use strict";
/**
 * Created by rburson on 3/18/16.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Smoothing techniques.
 * http://perfectionkills.com/exploring-canvas-drawing-techniques/
 */
var React = require("react");
var catreact_1 = require("catreact");
var catavolt_sdk_1 = require("catavolt-sdk");
/**
* Value to be assigned when CvSignaturePopup is mounted.
*/
var openSignaturePopupListener;
function openSignaturePopup(listener, signature, ratio) {
    if (openSignaturePopupListener != null) {
        if (signature) {
            var c = { background: signature.backgroundColor, line: signature.lineColor };
            var s = { width: signature.captureBounds.width.value, height: signature.captureBounds.height.value };
            openSignaturePopupListener({ listener: listener, colors: c, size: s, ratio: ratio });
        }
        else {
            // Provide some defaults
            var c = { background: new catavolt_sdk_1.Color(null, 255, 255, 255, 1), line: new catavolt_sdk_1.Color(null, 0, 0, 0, 1) };
            var s = { width: 800, height: 600 };
            var r = { width: 400, height: 200 };
            openSignaturePopupListener({ listener: listener, colors: c, size: s, ratio: r });
        }
    }
}
exports.openSignaturePopup = openSignaturePopup;
/**
 Component that overlays a signature popup.
 */
exports.CvSignaturePopup = React.createClass({
    componentWillMount: function () {
        var signatureProvider = new catreact_1.CvValueAdapter();
        signatureProvider.subscribe(this._handleParamChange);
        openSignaturePopupListener = signatureProvider.createValueListener();
        window.addEventListener("mouseup", this._onMouseUp);
    },
    componentWillUnmount: function () {
        window.removeEventListener("mouseup", this._onMouseUp);
        this.state.canvas = null;
    },
    getDefaultProps: function () {
        return { paramProvider: null };
    },
    getInitialState: function () {
        return {
            resultListener: null,
            forceRedraw: 1,
            drawingStatus: { isDrawing: false, someDrawn: false }
        };
    },
    render: function () {
        var _this = this;
        return (React.createElement("div", { className: "modal fade", role: "dialog", key: "CvSignaturePopup", ref: function (d) {
                var visible = (_this.state != null) && (_this.state.resultListener != null);
                if (visible) {
                    var m = $(d);
                    if (m && m.modal) {
                        m.modal({ show: true, keyboard: false, backdrop: 'static' });
                    }
                    var ctx = _this._get2dContext();
                    ctx.strokeStyle = catreact_1.ColorUtil.toColor(_this.state.lineColor);
                    ctx.lineWidth = 7;
                    ctx.lineCap = "round";
                    ctx.lineJoin = "round";
                    _this._resetDrawing();
                }
                else {
                    var m = $(d);
                    if (m && m.modal) {
                        m.modal('hide');
                    }
                }
            } },
            React.createElement("div", { className: "modal-dialog cv-signature" },
                React.createElement("div", { className: "modal-content " },
                    React.createElement("div", { className: "modal-header cv-signature-header" },
                        React.createElement("p", null, "Signature Capture")),
                    React.createElement("div", { className: "modal-body" },
                        React.createElement("canvas", { ref: function (c) { _this.state.canvas = c; }, className: "cv-signature-canvas", height: this.state.height, width: this.state.width, onMouseDown: this._onMouseDown, onMouseMove: this._onMouseMove })),
                    React.createElement("div", { className: "modal-footer cv-signature-footer" },
                        React.createElement("button", { type: "button", className: "btn btn-default", onClick: function () {
                                _this.state.resultListener(new catreact_1.CvImagePackageUrl(_this.state.canvas.toDataURL()));
                                _this.setState({ resultListener: null }); // Clearing the listener closes the popup
                            } }, "Save"),
                        React.createElement("button", { type: "button", className: "btn btn-default", onClick: function () {
                                _this._resetDrawing();
                            } }, "Redo"),
                        React.createElement("button", { type: "button", className: "btn btn-default", onClick: function () {
                                _this.setState({ resultListener: null });
                            } }, "Cancel"))))));
    },
    _get2dContext: function () {
        // https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D
        return this.state.canvas.getContext("2d");
    },
    _getMousePos: function (event) {
        var rect = this.state.canvas.getBoundingClientRect();
        return {
            x: event.clientX - rect.left,
            y: event.clientY - rect.top
        };
    },
    _handleParamChange: function (sigBundle) {
        if (sigBundle.listener) {
            var maxW = Math.min(sigBundle.size.width, $(window).innerWidth() - 50);
            var maxH = Math.min(sigBundle.size.height, $(window).innerHeight() - 200);
            var size = catreact_1.ImageUtil.calculateAspectRatioFit(sigBundle.ratio.width, sigBundle.ratio.height, maxW, maxH);
            this.setState({ backgroundColor: sigBundle.colors.background, lineColor: sigBundle.colors.line,
                width: size.width, height: size.height });
        }
        this.setState({ resultListener: sigBundle.listener });
    },
    _onMouseDown: function (event) {
        if (this.state.canvas) {
            catavolt_sdk_1.Log.debug("CvSignaturePopup:MouseDown");
            var ctx = this._get2dContext();
            ctx.beginPath();
            this.state.drawingStatus.isDrawing = true;
            this.state.drawingStatus.someDrawn = false;
            var pos = this._getMousePos(event);
            ctx.moveTo(pos.x, pos.y);
        }
    },
    _onMouseUp: function (event) {
        if (this.state.drawingStatus.isDrawing) {
            catavolt_sdk_1.Log.debug("CvSignaturePopup:MouseUp");
            var ctx = this._get2dContext();
            var pos = this._getMousePos(event);
            if (!this.state.drawingStatus.someDrawn) {
                // If just a mouse click, draw a dot.
                ctx.strokeStyle = catreact_1.ColorUtil.toColor(this.state.lineColor);
                ctx.lineTo(pos.x + 1, pos.y + 1);
                ctx.stroke();
            }
            ctx.closePath();
            this.state.drawingStatus.isDrawing = false;
            this.state.drawingStatus.someDrawn = false;
        }
    },
    _onMouseMove: function (event) {
        if (this.state.drawingStatus.isDrawing) {
            var pos = this._getMousePos(event);
            var ctx = this._get2dContext();
            ctx.strokeStyle = catreact_1.ColorUtil.toColor(this.state.lineColor);
            ctx.lineTo(pos.x, pos.y);
            ctx.stroke();
            ctx.moveTo(pos.x, pos.y);
            this.state.drawingStatus.someDrawn = true;
            catavolt_sdk_1.Log.debug("CvSignaturePopup:MouseMove  x: " + pos.x + "  y: " + pos.y);
        }
    },
    _resetDrawing: function () {
        var ctx = this._get2dContext();
        ctx.fillStyle = catreact_1.ColorUtil.toColor(this.state.backgroundColor);
        ctx.fillRect(0, 0, this.state.width, this.state.height);
    }
});
