"use strict";
/**
 * Created by rburson on 3/30/16.
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var catreact_1 = require("catreact");
var catavolt_sdk_1 = require("catavolt-sdk");
exports.CvViewMenuBase = {
    paneContext: function (nextProps, nextContext) {
        var paneContext = null;
        if (nextProps && nextProps.paneContext) {
            paneContext = nextProps.paneContext;
        }
        else if (nextContext) {
            paneContext = this.findPaneContext(nextContext.cvContext.scopeCtx);
        }
        else {
            paneContext = this.props.paneContext || this.findPaneContext();
        }
        return paneContext;
    },
    refresh: function (nextProps, nextContext) {
        var _this = this;
        /*
         * The purpose of this method is to reconcile the selectedView (given as a parameter)
          * with the server's selectedView
         */
        var paneContext = this.paneContext(nextProps, nextContext);
        var viewDescs = this.viewDescs(nextProps, nextContext);
        var selectedView = nextProps && nextProps.selectedViewId ? nextProps.selectedViewId : this.props.selectedViewId;
        var selectedViewDesc = catavolt_sdk_1.ArrayUtil.find(viewDescs, function (viewDesc) {
            return selectedView == viewDesc.viewId;
        });
        //if state hasn't been set, retrieve from server and set
        if (!this.state.selectedViewId) {
            paneContext.getSelectedViewId().onComplete(function (viewTry) {
                if (viewTry.isSuccess) {
                    if (_this.isMounted()) {
                        _this.setState({ selectedViewId: viewTry.success });
                    }
                    //compare 'requested' view with what the server 'thinks' is the selected view
                    if (selectedViewDesc && selectedViewDesc.viewId != viewTry.success.value) {
                        //try to open the 'requested' view
                        catreact_1.CvViewDesc.openView(selectedViewDesc, paneContext, _this.eventRegistry(), _this.catavolt(), _this.props.actionListeners, [], _this.props.navigationListeners, _this.props.navTarget, function (success, error) {
                            if (!error) {
                                _this.setState({ selectedViewId: new catavolt_sdk_1.ViewId(selectedViewDesc.viewId) });
                                _this.props.viewLoadedListener && _this.props.viewLoadedListener(selectedViewDesc.viewId);
                            }
                        });
                    }
                    else {
                        _this.props.viewLoadedListener && _this.props.viewLoadedListener(viewTry.success.value);
                    }
                }
                else {
                    var event_1 = {
                        type: catreact_1.CvEventType.MESSAGE,
                        eventObj: {
                            message: 'Could not get selected view id for ' + paneContext.paneDef.findTitle(),
                            messageObj: viewTry.failure,
                            type: catreact_1.CvMessageType.ERROR
                        }
                    };
                    _this.eventRegistry().publish(event_1, false);
                }
            });
        }
        else {
            this.props.viewLoadedListener && this.props.viewLoadedListener(this.state.selectedViewId.value);
        }
    },
    viewDescs: function (nextProps, nextContext) {
        if (nextProps && nextProps.viewDescs) {
            return nextProps.viewDescs;
        }
        else {
            return this.props.viewDescs;
        }
    }
};
/*
 ***************************************************
 * Render a Menu
 ***************************************************
 */
exports.CvDropdownViewMenu = React.createClass({
    mixins: [catreact_1.CvBaseMixin, exports.CvViewMenuBase],
    componentWillMount: function () {
        if (!this._checkVisibility()) {
            this.props.viewLoadedListener && this.props.viewLoadedListener('no_view_support');
        }
    },
    componentDidMount: function () {
        if (this.state.visible) {
            this.refresh();
        }
    },
    componentWillReceiveProps: function (nextProps, nextContext) {
        if (this._checkVisibility(nextProps, nextContext)) {
            this.refresh(nextProps, nextContext);
        }
    },
    getDefaultProps: function () {
        return {
            paneContext: null,
            actionListeners: [],
            navigationListeners: [],
            navTarget: null,
            viewDescs: [],
            openLeft: false,
            viewSelectionListener: null,
            viewLoadedListener: null,
            selectedViewId: null
        };
    },
    getInitialState: function () {
        return { selectedViewId: null, visible: true };
    },
    render: function () {
        var _this = this;
        if (this.state.visible) {
            var viewDescs = this.viewDescs();
            var selectedViewId_1 = this.state.selectedViewId;
            var buttonText = 'Select View';
            if (selectedViewId_1 && selectedViewId_1.value) {
                var selectedViewDesc = catavolt_sdk_1.ArrayUtil.find(this.viewDescs(), function (viewDesc) {
                    return selectedViewId_1.value == viewDesc.viewId;
                });
                if (selectedViewDesc) {
                    buttonText = selectedViewDesc.name;
                }
            }
            var itemProps_1 = {
                paneContext: this.props.paneContext,
                actionListeners: this.props.actionListeners,
                navigationListeners: this.props.navigationListeners,
                navTarget: this.props.navTarget,
                catavolt: this.props.catavolt,
                eventRegistry: this.props.eventRegistry
            };
            var viewItemClassName = "dropdown-menu cv-dropdown-menu";
            if (this.props.openLeft)
                viewItemClassName += ' dropdown-menu-right';
            var itemRenderer_1 = function (cvContext, callback) {
                var viewDesc = cvContext.scopeCtx.scopeObj;
                return React.createElement("li", { onClick: function () {
                        if (selectedViewId_1 && (selectedViewId_1.value != viewDesc.viewId)) {
                            callback.openView(function (success, error) {
                                if (!error) {
                                    _this.setState({ selectedViewId: new catavolt_sdk_1.ViewId(viewDesc.viewId) });
                                    _this.props.viewSelectionListener && _this.props.viewSelectionListener(viewDesc.viewId);
                                }
                            });
                        }
                    } },
                    React.createElement("a", { className: "cv-target" }, viewDesc.name));
            };
            return (React.createElement("span", { className: "dropdown cv-view-dropdown" },
                React.createElement("button", { className: "btn btn-xs", type: "button", "data-toggle": "dropdown", "aria-haspopup": "true", "aria-expanded": "true" },
                    buttonText,
                    " ",
                    React.createElement("span", { className: "caret" })),
                React.createElement("ul", { className: viewItemClassName }, viewDescs.map(function (viewDesc) {
                    return React.createElement(catreact_1.CvViewDesc, __assign({}, itemProps_1, { viewDesc: viewDesc, renderer: itemRenderer_1, key: viewDesc.viewId }));
                }))));
        }
        else {
            return null;
        }
    },
    _checkVisibility: function (nextProps, nextContext) {
        var viewDescs = this.viewDescs(nextProps, nextContext);
        if (!this.catavolt().isFeatureSetAvailable("View_Support") || !viewDescs || viewDescs.length < 2) {
            if (this.state.visible) {
                this.setState({ visible: false });
            }
            return false;
        }
        else {
            if (!this.state.visible) {
                this.setState({ visible: true });
            }
            return true;
        }
    }
});
